@@ -178,12 +178,11 @@ On Ubuntu or Debian install package 'python-coverage'.
 	mylib = os.path.abspath('./zim')
 	for module in [m for m in sys.modules.keys()
 			if m == 'zim' or m.startswith('zim.')]:
-		if sys.modules[module] is None:
-			continue
-		file = sys.modules[module].__file__
-		assert file.startswith(mylib), \
-			'Module %s was loaded from %s' % (module, file)
-
+				if sys.modules[module] is None:
+					continue
+				file = sys.modules[module].__file__
+				assert file.startswith(mylib), \
+					'Module %s was loaded from %s' % (module, file)
 
 	if coverage:
 		coverage.stop()
@@ -258,10 +257,12 @@ def report_coverage(coverage):
 <head>
 <title>Test Coverage Index</title>
 <style>
-	.good  { background-color: #9f9; text-align: right }
-	.close { background-color: #ff9; text-align: right }
-	.bad   { background-color: #f99; text-align: right }
-	.int   { text-align: right }
+	.good    { background-color: #9f9; text-align: right }
+	.close   { background-color: #cf9; text-align: right }
+	.ontrack { background-color: #ff9; text-align: right }
+	.lacking { background-color: #fc9; text-align: right }
+	.bad     { background-color: #f99; text-align: right }
+	.int     { text-align: right }
 </style>
 </head>
 <body>
@@ -273,8 +274,10 @@ def report_coverage(coverage):
 	total_stat = reduce(int.__add__, [r[2] for r in index])
 	total_exec = reduce(int.__add__, [r[3] for r in index])
 	total_perc = int( float(total_exec) / total_stat * 100 )
-	if total_perc == 100: type = 'good'
-	elif total_perc > 80: type = 'close'
+	if total_perc >= 90: type = 'good'
+	elif total_perc >= 80: type = 'close'
+	elif total_perc >= 60: type = 'ontrack'
+	elif total_perc >= 40: type = 'lacking'
 	else: type = 'bad'
 	html.write('<tr><td><b>Total</b></td>'
 		       '<td class="int">%i</td><td class="int">%i</td>'
@@ -285,8 +288,10 @@ def report_coverage(coverage):
 		pyfile, htmlfile, statements, executed = report
 		if statements: percentage = int( float(executed) / statements * 100 )
 		else: percentage = 100
-		if percentage == 100: type = 'good'
-		elif percentage > 80: type = 'close'
+		if percentage >= 90: type = 'good'
+		elif percentage >= 80: type = 'close'
+		elif percentage >= 60: type = 'ontrack'
+		elif percentage >= 40: type = 'lacking'
 		else: type = 'bad'
 		html.write('<tr><td><a href="%s">%s</a></td>'
 		           '<td class="int">%i</td><td class="int">%i</td>'
