@@ -8,18 +8,21 @@ import os
 import sys
 import shutil
 import unittest
+import logging
 import gettext
 import xml.etree.cElementTree as etree
 
 
 __all__ = [
-	'errors', 'parsing', 'fs', 'config', 'applications',
+	'coding', 'translations',
+	'errors', 'parsing', 'fs', 'config', 'applications', 'async',
 	'formats', 'templates',
 	'stores', 'index', 'notebook',
 	'history', 'plugins',
 	'export', 'www', 'search',
-	'pageview',
-	'calendar', 'printtobrowser', 'versioncontrol',
+	'widgets', 'gui', 'pageview',
+	'calendar', 'printtobrowser', 'versioncontrol', 'inlinecalculator',
+	'tasklist',
 	'equationeditor', 'diagrameditor',
 ]
 
@@ -32,6 +35,7 @@ gettext.install('zim', unicode=True, names=('_', 'gettext', 'ngettext'))
 def set_environ():
 	tmpdir = './tests/tmp/'
 	os.environ.update({
+		'ZIM_TEST_RUNNING': 'True',
 		'TMP': tmpdir,
 		'XDG_DATA_HOME': './tests/tmp/data_home',
 		'XDG_DATA_DIRS': './tests/tmp/data_dir',
@@ -47,6 +51,9 @@ def set_environ():
 		shutil.rmtree(tmpdir)
 	os.mkdir(tmpdir)
 
+	hicolor = os.environ['XDG_DATA_DIRS'] + '/icons/hicolor'
+	os.makedirs(hicolor)
+
 
 def create_tmp_dir(name):
 	'''Returns a path to a tmp dir for tests to store dump data.
@@ -75,6 +82,7 @@ def get_test_data(format):
 		_test_data_wiki = _get_test_data_wiki()
 
 	for name, text in _test_data_wiki:
+		#~ print '>', name
 		yield name, text
 
 
@@ -148,7 +156,7 @@ def print_index(index):
 
 
 class TestCase(unittest.TestCase):
-	'''FIXME'''
+	'''Base class for test cases'''
 
 	def run(self, *args):
 		unittest.TestCase.run(self, *args)
@@ -219,3 +227,61 @@ class TestCase(unittest.TestCase):
 			return
 
 		raise self.failureException, msg.encode('utf-8')
+
+
+class MockObject(object):
+	'''Base class for mock objects. Any attribute is automatically
+	vivicated as a method which results None. Alternatively mock
+	methods can be installed with mock_method(). Attributes that are
+	not methods need to be initialized explicitly.
+
+	All method call are logged, so they can be inspected. The attribute
+	'mock_calls' has a list of tuples with mock methods and arguments in
+	order they have been called.
+	'''
+
+	def __init__(self):
+		self.mock_calls = []
+
+	def __getattr__(self, name):
+		'''Automatically mock methods'''
+		return self.mock_method(name, None)
+
+	def mock_method(self, name, return_value):
+		'''Installs a mock method with a given name that returns
+		a given value.
+		'''
+		def my_mock_method(*arg, **kwarg):
+			call = [name] + list(arg)
+			if kwarg:
+				call.append(kwarg)
+			self.mock_calls.append(tuple(call))
+			return return_value
+
+		setattr(self, name, my_mock_method)
+		return my_mock_method
+
+
+class LoggingFilter(object):
+	'''Base class for logging filters that can be used as a context
+	using the "with" keyword. To subclass it you only need to set the
+	logger to be used and (the begin of) the message to filter.
+	'''
+
+	logger = None
+	message = None
+
+	def __init__(self, logger=None, message=None):
+		if logger: self.logger = logger
+		if message: self.message = message
+
+		self.loggerobj = logging.getLogger(self.logger)
+
+	def __enter__(self):
+		self.loggerobj.addFilter(self)
+
+	def __exit__(self, *a):
+		self.loggerobj.removeFilter(self)
+
+	def filter(self, record):
+		return not record.getMessage().startswith(self.message)
