@@ -2,8 +2,10 @@
 
 # Copyright 2008 Jaap Karssenberg <pardus@cpan.org>
 
+from __future__ import with_statement
+
 import tests
-from tests import TestCase
+from tests import TestCase, LoggingFilter
 
 import os
 
@@ -13,6 +15,12 @@ from zim.notebook import Path
 import zim.config
 
 
+class FilterInvalidConfigWarning(LoggingFilter):
+
+	logger = 'zim.config'
+	message = 'Invalid config value'
+
+
 class TestDirsTestSetup(TestCase):
 
 	def runTest(self):
@@ -165,11 +173,57 @@ none=None
 		conf.set_modified(False)
 		self.assertEqual(conf['Foo'].setdefault('foobar', 5), 0)
 		self.assertEqual(conf['Bar'].setdefault('check', 3.14), 1.333)
-		self.assertEqual(conf['Bar'].setdefault('check', None), 1.333)
-		self.assertEqual(conf['Bar'].setdefault('check', 'foo', klass=float), 1.333)
-		self.assertEqual(conf['Foo'].setdefault('tja', (3,4), check=conf.is_coord), (33,44))
+		self.assertEqual(conf['Bar'].setdefault('check', None, float), 1.333)
+		self.assertEqual(conf['Foo'].setdefault('tja', (3,4), value_is_coord), (33,44))
+		self.assertEqual(conf['Bar'].setdefault('hmmm', 'foo', set(('foo', 'tja'))), 'tja')
 		self.assertFalse(conf.modified)
 
+		conf['Foo']['tja'] = [33, 44]
+		conf.set_modified(False)
+		self.assertEqual(conf['Foo'].setdefault('tja', (3,4)), (33,44))
+		self.assertEqual(conf['Foo'].setdefault('tja', (3,4), tuple), (33,44))
+		self.assertFalse(conf.modified)
+
+		conf['Foo']['tja'] = [33, 44]
+		conf.set_modified(False)
+		self.assertEqual(conf['Foo'].setdefault('tja', (3,4), allow_empty=True), (33,44))
+		self.assertFalse(conf.modified)
+
+		conf.set_modified(False)
+		with FilterInvalidConfigWarning():
+			self.assertEqual(
+			conf['Bar'].setdefault('hmmm', 'foo', set(('foo', 'bar'))),
+			'foo')
+		self.assertTrue(conf.modified)
+
+		conf.set_modified(False)
+		with FilterInvalidConfigWarning():
+			self.assertEqual(conf['Bar'].setdefault('check', 10, int), 10)
+		self.assertTrue(conf.modified)
+
+		conf['Bar']['string'] = ''
+		conf.set_modified(False)
+		with FilterInvalidConfigWarning():
+			self.assertEqual(conf['Bar'].setdefault('string', 'foo'), 'foo')
+		self.assertTrue(conf.modified)
+
+		conf['Bar']['string'] = ''
+		conf.set_modified(False)
+		self.assertEqual(conf['Bar'].setdefault('string', 'foo', allow_empty=True), '')
+		self.assertFalse(conf.modified)
+
+		conf['Bar']['string'] = ''
+		conf.set_modified(False)
+		self.assertEqual(conf['Bar'].setdefault('string', 'foo', check_class_allow_empty), '')
+		self.assertFalse(conf.modified)
+
+		conf['Bar']['string'] = 3
+		conf.set_modified(False)
+		with FilterInvalidConfigWarning():
+			self.assertEqual(conf['Bar'].setdefault('string', 'foo', check_class_allow_empty), 'foo')
+		self.assertTrue(conf.modified)
+
+
 	def testLookup(self):
 		'''Test lookup of config files'''
 		home = XDG_CONFIG_HOME.file('zim/preferences.conf')
@@ -183,7 +237,7 @@ none=None
 		self.assertEqual(file.file, home)
 		self.assertEqual(file.default, default)
 		self.assertEqual(file['TestData']['file'], 'default')
-		
+
 		home.write('[TestData]\nfile=home\n')
 		file = config_file('preferences.conf')
 		self.assertTrue(isinstance(file, ConfigDictFile))
@@ -214,31 +268,57 @@ none=None
 		self.assertEqual(val, 'ja')
 		keys.append('dus')
 
+		self.assertEquals(mydict.keys(), keys)
+		self.assertEquals([k for k in mydict], keys)
+
 		mykeys = [k for k, v in mydict.items()]
 		self.assertEquals(mykeys, keys)
+		myval = [v for k, v in mydict.items()]
+		self.assertEquals(myval, ['dusss', 'dusss', 'dusss', 'ja'])
 
-	#~ def testConfigList(self):
-		#~ '''Test ConfigList class'''
-		#~ input = u'''\
-#~ foo	bar
-	#~ dusss ja
-#~ # comments get taken out
-#~ some\ space he\ re # even here
-#~ empty
-#~ '''
-		#~ output = u'''\
-#~ foo\tbar
-#~ dusss\tja
-#~ some\\ space\the\\ re
-#~ empty\t
-#~ '''
-		#~ keys = ['foo', 'dusss', 'some space', 'empty']
-		#~ mydict = ConfigList()
-		#~ mydict.parse(input)
-		#~ mykeys = [k for k, v in mydict.items()]
-		#~ self.assertEquals(mykeys, keys)
-		#~ result = mydict.dump()
-		#~ self.assertEqualDiff(result, output.splitlines(True))
+		val = mydict.pop('bar')
+		self.assertEqual(val, 'dusss')
+		self.assertEqual(mydict.keys(), ['foo', 'baz', 'dus'])
+
+		mydict.update({'bar': 'barrr'}, tja='ja ja')
+		self.assertEquals(mydict.items(), (
+			('foo', 'dusss'),
+			('baz', 'dusss'),
+			('dus', 'ja'),
+			('bar', 'barrr'),
+			('tja', 'ja ja'),
+		))
+
+		del mydict['tja']
+		self.assertEquals(mydict.items(), (
+			('foo', 'dusss'),
+			('baz', 'dusss'),
+			('dus', 'ja'),
+			('bar', 'barrr'),
+		))
+
+		mydict.update((('tja', 'ja ja'), ('baz', 'bazzz')))
+		self.assertEquals(mydict.items(), (
+			('foo', 'dusss'),
+			('baz', 'bazzz'),
+			('dus', 'ja'),
+			('bar', 'barrr'),
+			('tja', 'ja ja'),
+		))
+
+		newdict = mydict.copy()
+		self.assertTrue(isinstance(newdict, ListDict))
+		self.assertEquals(newdict.items(), mydict.items())
+
+		mydict.set_order(('baz', 'bar', 'foo', 'boooo', 'dus'))
+		self.assertEquals(mydict.items(), (
+			('baz', 'bazzz'),
+			('bar', 'barrr'),
+			('foo', 'dusss'),
+			('dus', 'ja'),
+			('tja', 'ja ja'),
+		))
+		self.assertTrue(isinstance(mydict.order, list))
 
 
 class TestHeaders(TestCase):
@@ -330,6 +410,8 @@ class TestHierarchicDict(TestCase):
 		dict = HierarchicDict()
 		dict['foo']['key1'] = 'foo'
 		self.assertEqual(dict['foo:bar:baz']['key1'], 'foo')
+		self.assertEqual(dict['foo:bar:baz'].get('key1'), 'foo')
+		self.assertEqual(dict['foo:bar:baz'].get('key7'), None)
 		dict['foo:bar']['key1'] = 'bar'
 		self.assertEqual(dict['foo:bar:baz']['key1'], 'bar')
 		self.assertEqual(dict['foo']['key1'], 'foo')
@@ -338,3 +420,4 @@ class TestHierarchicDict(TestCase):
 		self.assertEqual(dict[Path('foo:bar:baz')]['key1'], 'foo')
 		dict['']['key2'] = 'FOO'
 		self.assertEqual(dict[Path('foo:bar:baz')]['key2'], 'FOO')
+
