@@ -4,7 +4,9 @@
 
 '''Test cases for the zim.formats module.'''
 
-from tests import TestCase, get_test_data_page, get_test_page
+from __future__ import with_statement
+
+from tests import TestCase, get_test_data_page, get_test_page, LoggingFilter
 
 from zim.formats import *
 from zim.notebook import Link
@@ -160,13 +162,17 @@ LINKS: FooBar
 
 
 Let's try these bold, italic, underline and strike
-And some //verbatim//
+	And some //verbatim// with an indent halfway the paragraph
 And don't forget these: *bold*, /italic/ / * *^%#@#$#!@)_!)_
 
 A list
 * foo
 	* bar
 	* baz
+And an indented list
+	* foo
+		* bar
+		* baz
 
 And a checkbox list
 [ ] item 1
@@ -247,19 +253,20 @@ sunt in culpa qui officia deserunt mollit anim id est laborum.
 LINKS: <link href=":foo:bar">:foo:bar</link> <link href="./file.png">./file.png</link> <link href="file:///etc/passwd">file:///etc/passwd</link>
 LINKS: <link href="Foo">Foo</link><link href="Bar">Bar</link>
 </p>
-<p indent="1">Some indented
+<p><div indent="1">Some indented
 paragraphs go here ...
-</p>
+</div></p>
 
 <p><img src="./equation003.png" type="equation" />
 </p>
 
 <p>Let's try these <strong>bold</strong>, <emphasis>italic</emphasis>, <mark>underline</mark> and <strike>strike</strike>
-And some <code>//verbatim//</code>
-And don't forget these: *bold*, /italic/ / * *^%#@#$#!@)_!)_
+<div indent="1">And some <code>//verbatim//</code> with an indent halfway the paragraph
+</div>And don't forget these: *bold*, /italic/ / * *^%#@#$#!@)_!)_
 </p>
 <p>A list
-<ul><li bullet="*">foo</li><ul><li bullet="*"><strike>bar</strike></li><li bullet="*">baz</li></ul></ul></p>
+<ul><li bullet="*">foo</li><ul><li bullet="*"><strike>bar</strike></li><li bullet="*">baz</li></ul></ul>And an indented list
+<ul indent="1"><li bullet="*">foo</li><ul><li bullet="*"><strike>bar</strike></li><li bullet="*">baz</li></ul></ul></p>
 <p>And a checkbox list
 <ul><li bullet="unchecked-box">item 1</li><ul><li bullet="checked-box">sub item 1</li><ul><li bullet="*">Some normal bullet</li></ul><li bullet="xchecked-box">sub item 2</li><li bullet="unchecked-box">sub item 3</li></ul><li bullet="unchecked-box">item 2</li><li bullet="unchecked-box">item 3</li><ul><li bullet="xchecked-box">item FOOOOOO !</li></ul></ul></p>
 <p>----
@@ -394,9 +401,11 @@ LINKS: <a href="page://:foo:bar" title=":foo:bar">:foo:bar</a> <a href="file://.
 LINKS: <a href="page://Foo" title="Foo">Foo</a><a href="page://Bar" title="Bar">Bar</a><br>
 </p>
 
-<p style='padding-left: 30pt'>
+<p>
+<div style='padding-left: 30pt'>
 Some indented<br>
 paragraphs go here ...<br>
+</div>
 </p>
 
 
@@ -407,7 +416,9 @@ paragraphs go here ...<br>
 
 <p>
 Let's try these <strong>bold</strong>, <em>italic</em>, <u>underline</u> and <strike>strike</strike><br>
-And some <code>//verbatim//</code><br>
+<div style='padding-left: 30pt'>
+And some <code>//verbatim//</code> with an indent halfway the paragraph<br>
+</div>
 And don't forget these: *bold*, /italic/ / * *^%#@#$#!@)_!)_<br>
 </p>
 
@@ -420,6 +431,14 @@ A list<br>
 <li>baz</li>
 </ul>
 </ul>
+And an indented list<br>
+<ul style='padding-left: 30pt'>
+<li>foo</li>
+<ul>
+<li><strike>bar</strike></li>
+<li>baz</li>
+</ul>
+</ul>
 </p>
 
 <p>
@@ -462,23 +481,30 @@ That's all ...<br>
 		self.assertEqualDiff(output, html.splitlines(True))
 
 
+class LatexLoggingFilter(LoggingFilter):
+
+	logger = 'zim.formats.latex'
+	message = 'No document type set in template'
+
+
 class TestLatexFormat(TestCase):
 
 	def testEncode(self):
 		'''test the escaping of certain characters'''
 		format = get_format('latex')
 
-		input = r'\foo $ % ^ \% bar'
-		wanted = r'$\backslash$foo \$  \% \^{} $\backslash$\% bar'
+		input = r'\foo $ % ^ \% bar < >'
+		wanted = r'$\backslash$foo \$  \% \^{} $\backslash$\% bar \textless \textgreater'
 		self.assertEqual(format.tex_encode(input), wanted)
 
 	def testExport(self):
 		'''test the export of a wiki page to latex'''
-		format = get_format('LaTeX')
-		testpage = get_test_data_page('wiki','Test:wiki')
-		tree = get_format('wiki').Parser().parse(testpage)
-		output = format.Dumper(linker=StubLinker()).dump(tree)
-		self.assertTrue('\chapter{Foo Bar}\n' in output)
+		with LatexLoggingFilter():
+			format = get_format('LaTeX')
+			testpage = get_test_data_page('wiki','Test:wiki')
+			tree = get_format('wiki').Parser().parse(testpage)
+			output = format.Dumper(linker=StubLinker()).dump(tree)
+			self.assertTrue('\chapter{Foo Bar}\n' in output)
 
 		# TODO test template_options.document_type
 
@@ -531,8 +557,7 @@ baz
 
 dus
 <pre>ja
-</pre>
-hmm
+</pre>hmm
 
 <h level="2">foo</h>
 bar
