@@ -2,17 +2,14 @@
 
 # Copyright 2008 Jaap Karssenberg <pardus@cpan.org>
 
-'''FIXME'''
-
 from tests import TestCase, get_test_notebook
 
 import zim.history
-from zim.history import History, HistoryRecord
+from zim.history import History, HistoryPath
 from zim.notebook import Path
 from zim.config import ConfigDict
 
 class TestHistory(TestCase):
-	'''FIXME'''
 
 	def setUp(self):
 		zim.history.MAX_HISTORY = 100
@@ -22,7 +19,7 @@ class TestHistory(TestCase):
 
 	def _assertCurrent(self, history, page):
 		current = history.get_current()
-		self.assertTrue(isinstance(current, HistoryRecord))
+		self.assertTrue(isinstance(current, HistoryPath))
 		self.assertEqual(current.name, page.name)
 
 	def testLinear(self):
@@ -32,6 +29,7 @@ class TestHistory(TestCase):
 		for page in self.pages:
 			history.append(page)
 		self.assertEqual(len(history.history), len(self.pages))
+		self.assertEqual(history.current, len(self.pages) - 1)
 
 		self._assertCurrent(history, self.pages[-1])
 
@@ -39,6 +37,12 @@ class TestHistory(TestCase):
 		self.assertEqual(pages[0], history.get_current())
 		self.assertEqual(len(pages), len(self.pages))
 
+		self.assertEqual(pages[0].cursor, None)
+			# Newly appended pages should not have the cursor
+			# set - pageview has logic to do the right thing when
+			# no cursor is set. Setting default e.g. 0 will
+			# overrule this logic.
+
 		# walk backwards
 		for i in range(2, len(self.pages)+1):
 			prev = history.get_previous()
@@ -63,6 +67,34 @@ class TestHistory(TestCase):
 		self.assertTrue(history.get_next() is None)
 		self.assertTrue(history.get_current().is_last)
 
+		# Add page multiple times
+		i = len(history.history)
+		path = Path(history.history[-1].name)
+		for j in range(5):
+			history.append(path)
+		self.assertEqual(len(history.history), i)
+		self.assertEqual(history.current, i - 1)
+
+		# Test dropping forward stack
+		path1 = history.history[10]
+		path2 = history.history[-1]
+		history.set_current(path1)
+		self.assertEqual(history.current, 10)
+		self.assertEqual(len(history.history), len(self.pages))
+
+		history.append(path2)
+		self.assertEqual(history.current, 11)
+		self.assertEqual(len(history.history), 12)
+
+		# Test max entries
+		old = zim.history.MAX_HISTORY
+		zim.history.MAX_HISTORY = 3
+		for page in self.pages:
+			history.append(page)
+		zim.history.MAX_HISTORY = old
+
+		self.assertEqual(len(history.history), 3)
+
 	def testUnique(self):
 		'''Get recent pages from history'''
 		history = History(self.notebook)
@@ -101,6 +133,62 @@ class TestHistory(TestCase):
 		self.assertEqual(history.get_child(Path('Test')), Path('Test:wiki'))
 		self.assertEqual(history.get_grandchild(Path('Test')), Path('Test:wiki'))
 
+	def testMovePage(self):
+		'''Test history is updated for moved pages'''
+		history = History(self.notebook)
+		for page in self.pages:
+			history.append(page)
+
+		self.assertTrue(Path('Test:wiki') in history.history)
+
+		history._on_page_moved(Path('Test'), Path('New'), False)
+		self.assertFalse(Path('Test:wiki') in history.history)
+		self.assertTrue(Path('New:wiki') in history.history)
+
+		history._on_page_moved(Path('New'), Path('Test'), False)
+		self.assertFalse(Path('New:wiki') in history.history)
+		self.assertTrue(Path('Test:wiki') in history.history)
+
+		self.assertEqual(history.history, self.pages)
+
+	def testDeletedNotInUnique(self):
+		'''Test if deleted pages and their children show up in unique history list'''
+		history = History(self.notebook)
+		for page in self.pages:
+			history.append(page)
+		for page in self.pages:
+			history.append(page)
+
+		self.assertEqual(len(history.history), 2 * len(self.pages))
+
+		uniques = list(history.get_unique())
+		self.assertEqual(len(uniques), len(self.pages))
+
+		history._on_page_deleted(self.pages[0])
+		uniques = list(history.get_unique())
+		self.assertTrue(len(uniques) < len(self.pages))
+		i = len(uniques)
+
+		history._on_page_stored(self.pages[0])
+		uniques = list(history.get_unique())
+		self.assertEqual(len(uniques), i + 1)
+			# Not same as len(self.pages) because of deleted children
+
+		for page in self.pages:
+			history._on_page_deleted(page)
+		uniques = list(history.get_unique())
+		self.assertEqual(len(uniques), 0)
+
+		self.assertEqual(
+			len(list(history.get_history())),
+			2 * len(self.pages)  )
+
+		for page in self.pages:
+			history._on_page_stored(page)
+		uniques = list(history.get_unique())
+		self.assertEqual(len(uniques), len(self.pages))
+
+
 	def testSerialize(self):
 		'''Test parsing the history from the state file'''
 		uistate = ConfigDict()
@@ -119,8 +207,7 @@ class TestHistory(TestCase):
 		# check state
 		#~ import pprint
 		#~ pprint.pprint(uistate)
-		self.assertEqual(len(uistate['History']['history']), len(history.history))
-		#~ self.assertEqual(len(uistate['History']['pages']), len(history.history))
+		self.assertEqual(len(uistate['History']['list']), len(history.history))
 		self.assertEqual(uistate['History']['current'], len(history.history)-3)
 
 		# clone uistate by text
@@ -129,8 +216,7 @@ class TestHistory(TestCase):
 		newuistate.parse(lines)
 
 		# check new state
-		self.assertEqual(len(uistate['History']['history']), len(history.history))
-		#~ self.assertEqual(len(newuistate['History']['pages']), len(history.history))
+		self.assertEqual(len(uistate['History']['list']), len(history.history))
 		self.assertEqual(newuistate['History']['current'], len(history.history)-3)
 
 		# and compare resulting history object
