@@ -37,13 +37,15 @@ class TestBazaar(TestCase):
 
 	@classmethod
 	def skipTest(klass):
-		if BazaarVCS.check_dependencies():
-			return False
+		if not BazaarVCS.check_dependencies():
+			return 'Missing dependencies'
 		else:
-			return 'Bazaar not found'
+			return False
 
 	def runTest(self):
 		'''Test Bazaar version control'''
+		print '\n!! Some raw output from Bazaar expected here !!'
+
 		root = get_tmp_dir('versioncontrol_TestBazaar')
 		vcs = BazaarVCS(root)
 		vcs.init()
@@ -101,7 +103,7 @@ added:
 		self.assertEqual(vcs.get_diff(), ['=== No Changes\n'])
 
 		file.write('foo\nbaz\n')
-		vcs.commit('test 2')
+		vcs.commit_async('test 2')
 		diff = vcs.get_diff(versions=(1, 2))
 		diff = ''.join(filter(ignorelines, diff))
 		self.assertEqualDiff(diff, '''\
