@@ -2,6 +2,7 @@
 
 import sys
 import logging
+import os
 
 # Check if we run the correct python version
 try:
@@ -12,6 +13,16 @@ except:
 	print >> sys.stderr, 'ERROR: zim needs python >= 2.5   (but < 3.0)'
 	sys.exit(1)
 
+# Win32: must setup log file or it tries to write to $PROGRAMFILES
+if os.name == "nt" and sys.argv[0].endswith('.exe'):
+	import tempfile
+	dir = tempfile.gettempdir()
+	if not os.path.isdir(dir):
+		os.makedirs(dir)
+	err_stream = open(dir + "\\zim.log", "w")
+	sys.stdout = err_stream
+	sys.stderr = err_stream
+
 # Preliminary initalization of logging because modules can throw warnings at import
 logging.basicConfig(level=logging.WARNING, format='%(levelname)s: %(message)s')
 
