@@ -3,7 +3,7 @@
 # Copyright 2008 Jaap Karssenberg <pardus@cpan.org>
 
 # Bunch of meta data, used at least in the about dialog
-__version__ = '0.48'
+__version__ = '0.50'
 __url__='http://www.zim-wiki.org'
 __author__ = 'Jaap Karssenberg <pardus@cpan.org>'
 __copyright__ = 'Copyright 2008 - 2010 Jaap Karssenberg <pardus@cpan.org>'
@@ -53,7 +53,7 @@ commands = ('help', 'version', 'gui', 'server', 'export', 'index', 'manual', 'pl
 commandopts = {
 	'gui': ('list', 'geometry=', 'fullscreen', 'no-daemon'),
 	'server': ('port=', 'template=', 'gui', 'no-daemon'),
-	'export': ('format=', 'template=', 'output=', 'root-url='),
+	'export': ('format=', 'template=', 'output=', 'root-url=', 'index-page='),
 	'index': ('output=',),
 	'plugin': (),
 	'daemon': (),
@@ -108,6 +108,7 @@ Export Options:
   --template      name of the template to use
   -o, --output    output directory
   --root-url      url to use for the document root
+  --index-page    index page name
 
   You can use the export option to print a single page to stdout.
   When exporting a whole notebook you need to provide a directory.
@@ -135,8 +136,10 @@ def main(argv):
 
 	# FIXME - this returns python.exe on my windows test
 	ZIM_EXECUTABLE = argv[0]
-	if '/' in ZIM_EXECUTABLE or '\\' in ZIM_EXECUTABLE:
-		ZIM_EXECUTABLE = File(ZIM_EXECUTABLE).path # abs path
+	zim_exec_file = File(ZIM_EXECUTABLE)
+	if zim_exec_file.exists():
+		# We were given an absolute path, e.g. "python ./zim.py"
+		ZIM_EXECUTABLE = zim_exec_file.path
 
 	# Let getopt parse the option list
 	short = ''.join(shortopts.keys())
@@ -358,14 +361,18 @@ class NotebookInterface(gobject.GObject):
 
 	def load_plugins(self):
 		'''Load the plugins defined in the preferences'''
-		self.preferences['General'].setdefault('plugins',
-			['calendar', 'insertsymbol', 'printtobrowser', 'versioncontrol'])
+		default = ['calendar', 'insertsymbol', 'printtobrowser', 'versioncontrol']
+		self.preferences['General'].setdefault('plugins', default)
 		plugins = self.preferences['General']['plugins']
 		plugins = set(plugins) # Eliminate doubles
 		# Plugins should not have dependency on order of being added
 		# just add sort to make behavior predictable.
-		for plugin in sorted(plugins):
-			self.load_plugin(plugin)
+		for name in sorted(plugins):
+			self.load_plugin(name)
+
+		loaded = [p.plugin_key for p in self.plugins]
+		if set(loaded) != plugins:
+			self.preferences['General']['plugins'] = sorted(loaded)
 
 	def load_plugin(self, name):
 		'''Load a single plugin by name, returns boolean for success'''
@@ -386,7 +393,6 @@ class NotebookInterface(gobject.GObject):
 		plugin.plugin_key = name
 		if not name in self.preferences['General']['plugins']:
 			self.preferences['General']['plugins'].append(name)
-			self.preferences.write()
 
 		return plugin
 
@@ -450,10 +456,10 @@ class NotebookInterface(gobject.GObject):
 				notebook.cache_dir.file('state.conf') )
 		# TODO read profile preferences file if one is set in the notebook
 
-	def cmd_export(self, format='html', template=None, page=None, output=None, root_url=None):
+	def cmd_export(self, format='html', template=None, page=None, output=None, root_url=None, index_page=None):
 		'''Method called when doing a commandline export'''
 		import zim.exporter
-		exporter = zim.exporter.Exporter(self.notebook, format, template, document_root_url=root_url)
+		exporter = zim.exporter.Exporter(self.notebook, format, template, document_root_url=root_url, index_page=index_page)
 
 		if page:
 			path = self.notebook.resolve_path(page)
