@@ -12,13 +12,14 @@ import sys
 import os
 import re
 import logging
+import types
 
 if sys.version_info >= (2, 6):
 	import json # in standard lib since 2.6
 else:
 	import simplejson as json # extra dependency
 
-from zim.fs import isfile, isdir, Dir, FileNotFoundError
+from zim.fs import isfile, isdir, File, Dir, FileNotFoundError
 from zim.errors import Error
 from zim.parsing import TextBuffer, split_quoted_strings
 
@@ -223,6 +224,45 @@ def user_dirs():
 	return dirs
 
 
+def check_class_allow_empty(value, default):
+	'''Check function for setdefault() which ensures the value is of
+	the same class as the default but allows it to be empty. THis is
+	the same as the default behavior when "allow_empty" is True.
+
+	Only reason to use this function is for places where setdefault()
+	is called indirectly, e.g. with arguments from plugin preferences.
+	'''
+	klass = default.__class__
+	if issubclass(klass, basestring):
+		klass = basestring
+
+	if value in ('', None) or isinstance(value, klass):
+		return value
+	elif klass is tuple and isinstance(value, list):
+		# Special case because json does not know difference list or tuple
+		return tuple(value)
+	else:
+		raise AssertionError, 'should be of type: %s' % klass
+
+
+def value_is_coord(value, default):
+	'''Check function for setdefault() which enforces a coordinate
+	(a tuple or list of 2 ints).
+	'''
+	if isinstance(value, list):
+		value = tuple(value)
+
+	if (
+		isinstance(value, tuple)
+		and len(value) == 2
+		and isinstance(value[0], int)
+		and isinstance(value[1], int)
+	):
+		return value
+	else:
+		raise AssertionError, 'should be coordinate (tuple of int)'
+
+
 class ListDict(dict):
 	'''Class that behaves like a dict but keeps items in same order.
 	Used as base class for e.g. for config objects were writing should be
@@ -233,6 +273,12 @@ class ListDict(dict):
 		self.order = []
 		self._modified = False
 
+	def copy(self):
+		'''Shallow copy'''
+		new = self.__class__()
+		new.update(self)
+		return new
+
 	@property
 	def modified(self):
 		'''Recursive property'''
@@ -268,70 +314,131 @@ class ListDict(dict):
 		dict.__delitem__(self, k)
 		self.order.remove(k)
 
+	def __iter__(self):
+		return iter(self.order)
+
 	def pop(self, k):
 		v = dict.pop(self, k)
 		self.order.remove(k)
+		return v
 
 	# Would expect that setdefault() triggers __setitem__
 	# but this seems not to be the case in the standard implementation
 	# And we added some extra functionality here
-	def setdefault(self, k, v=None, klass=None, check=None):
-		'''Like dict.setdefault() but with some extra restriction because we
-		assume un-safe user input. If 'klass' is given the existing value
-		will be checked to be of that class and reset to default if it is not.
-		Alternatively 'check' can be a function that needs to return True
-		in order to keep the existing value. If no class and no function
-		is given is it will compare the classes of the set value and the
-		default to ensure we get what we expect. (An exception is made when
-		value is None, in that case it is good practise to always specify
-		a class or check function.)
-		'''
-		if not k in self:
-			self.__setitem__(k, v)
-		elif check:
-			if not check(self[k]):
-				logger.warn(
-					'Invalid config value for %s: "%s"', k, self[k])
-				self.__setitem__(k, v)
-		else:
-			if klass is None:
-				if v is None:
-					return self[k] # noting we can do
-				else:
-					klass = v.__class__
+	def setdefault(self, key, default, check=None, allow_empty=False):
+		'''Like dict.setdefault() but with some extra restriction
+		because we assume un-safe user input. If no extra arguments
+		are given it will compare the classes of the set value and the
+		default to ensure we get what we expect. An exception is made
+		when value is None, in that case it is good practise to always
+		specify a class or check function. When the default is a string
+		we check the value to be an instance of basestring (ignoring
+		difference between str and unicode). Another special case is when
+		the default is a tuple and the value is a list, in this case the
+		value will be cast to a tuple.
 
+		If 'check' is given and is a class the existing value will be
+		checked to be of that class and reset to default if it is not.
+		Same spacial case for tuples applies here.
+
+		If 'check' is given and it is a function it will be used to
+		check the value in the dictionary if it exists. The check
+		function gets the current value and the default value as
+		arguments. The function should raise an AssertionError when the
+		value is not ok. The return value of the function is used to
+		replace the current value, so the check function can coerce
+		values into the proper form (but don't use this to return a
+		default!).
+		( Note that 'assert' statements in the code can be removed
+		by code optimization, so explicitly call 'raise' to raise the
+		AssertionError. )
+
+		If 'check' is given and is a set the value will be tested
+		against this set.
+
+		If 'allow_empty' is True values are also allowed to be empty
+		string or None. This is used for optional parameters in the
+		config. By default 'allow_empty' is False but it is set to
+		True implicitely when the default value is None or ''.
+		'''
+		if not key in self:
+			self.__setitem__(key, default)
+			return self[key]
+
+		if check is None:
+			assert not default is None, 'Bad practise to set default to None without check'
+			klass = default.__class__
 			if issubclass(klass, basestring):
 				klass = basestring
+			check = klass
 
-			if not (self[k] is None and v is None) \
-			and not isinstance(self[k], klass):
-				if klass is tuple and isinstance(self[k], list):
+		if default in ('', None):
+			allow_empty = True
+
+		if allow_empty and self[key] in ('', None):
+			return self[key]
+
+		if isinstance(check, (type, types.ClassType)): # is a class
+			klass = check
+			if not (allow_empty and default in ('', None)):
+				assert isinstance(default, klass), 'Default does not have correct class'
+
+			if not isinstance(self[key], klass):
+				if klass is tuple and isinstance(self[key], list):
 					# Special case because json does not know difference list or tuple
-					v = tuple(self[k])
-					self.__setitem__(k, v)
+					modified = self.modified
+					self.__setitem__(key, tuple(self[key]))
+					self.set_modified(modified) # don't change modified state
 				else:
 					logger.warn(
 						'Invalid config value for %s: "%s" - should be of type %s',
-						k, self[k], klass)
-					self.__setitem__(k, v)
-			elif klass is basestring and v and not self[k]:
-				# Special case to avoid empty string
-				self.__setitem__(k, v)
+						key, self[key], klass)
+					self.__setitem__(key, default)
+			elif self[key] == '':
+					# Special case for empty string
+					logger.warn(
+						'Invalid config value for %s: "%s" - not allowed to be empty',
+						key, self[key])
+					self.__setitem__(key, default)
+			else:
+				pass # value is OK
+		elif isinstance(check, set):
+			if not (allow_empty and default in ('', None)):
+				assert default in check, 'Default is not within allows set'
 
-		return self[k]
+			if not self[key] in check:
+				logger.warn(
+						'Invalid config value for %s: "%s" - should be one of %s',
+						key, self[key], unicode(check))
+				self.__setitem__(key, default)
+			else:
+				pass # value is OK
+		else: # assume callable
+			modified = self.modified
+			try:
+				v = check(self[key], default)
+				self.__setitem__(key, v)
+				self.set_modified(modified)
+			except AssertionError, error:
+				logger.warn(
+					'Invalid config value for %s: "%s" - %s',
+					key, self[key], error.args[0])
+				self.__setitem__(key, default)
+
+		return self[key]
 
 	def keys(self):
 		return self.order[:]
 
 	def items(self):
-		return map(lambda k: (k, self[k]), self.order)
+		return tuple(map(lambda k: (k, self[k]), self.order))
 
 	def set_order(self, order):
 		'''Change the order in which items are listed by setting a list
 		of keys. Keys not in the list are moved to the end. Keys that are in
 		the list but not in the dict will be ignored.
 		'''
-		order = order[:] # copy
+		order = list(order[:]) # copy and convert
 		oldorder = set(self.order)
 		neworder = set(order)
 		for k in neworder - oldorder: # keys not in the dict
@@ -342,36 +449,6 @@ class ListDict(dict):
 		assert neworder == oldorder
 		self.order = order
 
-	def check_is_int(self, key, default):
-		'''Asserts that the value for 'key' is an int. If this is not
-		the case or when no value is set at all for 'key'.
-		'''
-		if not key in self:
-			self[key] = default
-		elif not isinstance(self[key], int):
-			logger.warn('Invalid config value for %s: "%s" - should be an integer')
-			self[key] = default
-
-	def check_is_float(self, key, default):
-		'''Asserts that the value for 'key' is a float. If this is not
-		the case or when no value is set at all for 'key'.
-		'''
-		if not key in self:
-			self[key] = default
-		elif not isinstance(self[key], float):
-			logger.warn('Invalid config value for %s: "%s" - should be a decimal number')
-			self[key] = default
-
-	@staticmethod
-	def is_coord(value):
-		'''Returns True if value is a coordinate (a tuple or list of 2 ints).
-		Can be used in combination with setdefault() to enforce data types.
-		'''
-		return (isinstance(value, (tuple, list))
-				and len(value) == 2
-				and isinstance(value[0], int)
-				and isinstance(value[1], int)  )
-
 
 class ConfigDict(ListDict):
 	'''Config object which wraps a dict of dicts.
@@ -386,6 +463,10 @@ class ConfigDict(ListDict):
 		return dict.__getitem__(self, k)
 
 	def parse(self, text):
+		'''Parse 'text' and set values based on this input'''
+		# Note that we explicitly do _not_ support comments on the end
+		# of a line. This is because "#" could be a valid character in
+		# a config value.
 		if isinstance(text, basestring):
 			text = text.splitlines(True)
 		section = None
@@ -430,6 +511,9 @@ class ConfigDict(ListDict):
 			return json.loads('"%s"' % value.replace('"', '\\"')) # force string
 
 	def dump(self):
+		'''Returns a list of lines with text representation of the
+		dict. Used to write as a config file.
+		'''
 		lines = []
 		for section, parameters in self.items():
 			if parameters:
@@ -445,9 +529,8 @@ class ConfigDict(ListDict):
 		elif value is True: return 'True'
 		elif value is False: return 'False'
 		elif value is None: return 'None'
-		elif isinstance(value, object) and value.__class__.__name__.endswith('Path'):
-			# Hack to avoid importing Path here to test isinstance
-			return value.name
+		elif hasattr(value, 'serialize_zim_config'):
+			return value.serialize_zim_config()
 		else:
 			return json.dumps(value, separators=(',',':'))
 				# specify separators for compact encoding
@@ -621,8 +704,9 @@ class HierarchicDict(object):
 
 	__slots__ = ('dict')
 
-	def __init__(self):
+	def __init__(self, defaults=None):
 		self.dict = {}
+		self.dict['__defaults__'] = defaults or {}
 
 	def __getitem__(self, k):
 		if not isinstance(k, basestring):
@@ -660,7 +744,10 @@ class HierarchicDictFrame(object):
 			if key in self.dict and k in self.dict[key]:
 				return self.dict[key][k]
 		else:
-			raise KeyError
+			if k in self.dict['__defaults__']:
+				return self.dict['__defaults__'][k]
+			else:
+				raise KeyError
 
 	def __setitem__(self, k, v):
 		if not self.key in self.dict:
