@@ -9,7 +9,7 @@ import logging
 from zim.fs import *
 from zim.config import data_file
 from zim.formats import get_format, BaseLinker
-from zim.templates import get_template
+from zim.templates import get_template, Template
 from zim.notebook import Path, Page, IndexPage, PageNameError
 from zim.stores import encode_filename
 from zim.parsing import url_encode
@@ -43,7 +43,7 @@ class Exporter(object):
 		else:
 			self.format = format
 
-		if template and isinstance(template, basestring):
+		if template and not isinstance(template, Template):
 			self.template = get_template(format, template)
 		else:
 			self.template = template
@@ -212,18 +212,20 @@ class StaticLinker(BaseLinker):
 			if parent == self.path:
 				path = './' + self.path.basename + '/'
 				downpath = page.relname(parent)
-				path += encode_filename(downpath) + self._extension
+				path += downpath
 			elif parent == page:
 				uppath = self.path.relname(parent)
 				path = '../' * (uppath.count(':') + 1)
-				path += encode_filename(page.basename) + self._extension
+				path += page.basename
 			else:
 				uppath = self.path.relname(parent)
 				downpath = page.relname(parent)
 				path = '../' * uppath.count(':') or './'
-				path += encode_filename(downpath) + self._extension
+				path += downpath
+
+			path = encode_filename(path) + self._extension
 			#~ print '>>>', path
-			return url_encode(path)
+			return url_encode(path.replace(' ', '_'))
 
 	def file(self, link):
 		if self.document_root_url and link.startswith('/'):
