@@ -188,13 +188,15 @@ class ParseTree(ElementTreeModule.ElementTree):
 		'''
 		root = self.getroot()
 		children = root.getchildren()
+		tail = "\n"
 		if children:
 			first = children[0]
 			if first.tag == 'h' and first.attrib['level'] >= level:
+				tail = first.tail # Keep trailing text
 				root.remove(first)
 		heading = Element('h', {'level': level})
 		heading.text = text
-		heading.tail = "\n"
+		heading.tail = tail
 		root.insert(0, heading)
 
 	def cleanup_headings(self, offset=0, max=6):
@@ -279,6 +281,19 @@ class ParseTree(ElementTreeModule.ElementTree):
 
 		return count
 
+	def get_ends_with_newline(self):
+		'''Checks whether this tree ends in a newline or not'''
+		newline = False
+		for element in self.getiterator():
+			if element.tail:
+				newline = element.tail.endswith('\n')
+			elif element.tag in ('li', 'h'):
+				newline = True
+			elif element.text:
+				newline = element.text.endswith('\n')
+
+		return newline
+
 
 count_eol_re = re.compile(r'\n+\Z')
 split_para_re = re.compile(r'((?:^[ \t]*\n){2,})', re.M)
@@ -411,14 +426,14 @@ class ParseTreeBuilder(object):
 			self._seen_eol = need_eol
 
 		# Fix prefix newlines
-		if self._tail and self._last.tag in ('h', 'p', 'pre') \
+		if self._tail and self._last.tag in ('h', 'p') \
 		and not text.startswith('\n'):
 			if text:
 				text = '\n' + text
 			else:
 				text = '\n'
 				self._seen_eol = 1
-		if self._tail and self._last.tag == 'li' \
+		elif self._tail and self._last.tag == 'li' \
 		and text.startswith('\n'):
 			text = text[1:]
 			if not text.strip('\n'):
