@@ -58,7 +58,7 @@ sectioning = {
 
 
 
-encode_re = re.compile(r'(\&|\$|\^|\%|\#|\_|\\)')
+encode_re = re.compile(r'(\&|\$|\^|\%|\#|\_|\\|\<|\>)')
 encode_dict = {
 	'\\': '$\\backslash$',
 	'&': '\\$',
@@ -67,6 +67,8 @@ encode_dict = {
 	'%': '\\%',
 	'#': '\\# ',
 	'_': '\\_',
+	'>': '\\textgreater',
+	'<': '\\textless',
 }
 
 
@@ -102,7 +104,7 @@ class Dumper(DumperClass):
 
 		for element in list.getchildren():
 			text = tex_encode(element.text)
-			if element.tag == 'p':
+			if element.tag in ('p', 'div'):
 				if 'indent' in element.attrib:
 					indent = int(element.attrib['indent'])
 				else:
@@ -139,9 +141,9 @@ class Dumper(DumperClass):
 				myoutput.append(element.text)
 				if indent:
 					myoutput.prefix_lines('    ' * indent)
-				output.append('\n\\begin{verbatim}\n')
+				output.append('\n\\begin{lstlisting}\n')
 				output.extend(myoutput)
-				output.append('\n\\end{verbatim}\n')
+				output.append('\n\\end{lstlisting}\n')
 			elif element.tag == 'sub':
 				output.append('$_{%s}$' % element.text)
 			elif element.tag == 'sup':
@@ -188,7 +190,7 @@ class Dumper(DumperClass):
 			elif element.tag == 'strong':
 				output.append('\\textbf{'+text+'}')
 			elif element.tag == 'mark':
-				output.append('\\underline{'+text+'}')
+				output.append('\\uline{'+text+'}')
 			elif element.tag == 'strike':
 				output.append('\\sout{'+text+'}')
 			elif element.tag == 'code':
@@ -197,7 +199,7 @@ class Dumper(DumperClass):
 				for delim in '+*|$&%!-_':
 					if not delim in text:
 						success = True
-						output.append('\\verb'+delim+text+delim)
+						output.append('\\lstinline'+delim+text+delim)
 						break
 				if not success:
 					assert False, 'Found no suitable delimiter for verbatim text: %s' % element
