@@ -38,18 +38,21 @@ from zim.notebook import Path, Page, PageNameError, \
 	resolve_default_notebook, get_notebook, get_notebook_list
 from zim.stores import encode_filename
 from zim.index import LINK_DIR_BACKWARD
-from zim.config import data_file, config_file, data_dirs, ListDict
+from zim.config import data_file, config_file, data_dirs, ListDict, value_is_coord
 from zim.parsing import url_encode, URL_ENCODE_DATA, is_win32_share_re
-from zim.history import History, HistoryRecord
+from zim.history import History, HistoryPath
+from zim.templates import list_templates, get_template
 from zim.gui.pathbar import NamespacePathBar, RecentPathBar, HistoryPathBar
 from zim.gui.pageindex import PageIndex
 from zim.gui.pageview import PageView
 from zim.gui.widgets import ui_environment, gtk_window_set_default_icon, \
 	Button, MenuButton, \
 	Window, Dialog, \
-	ErrorDialog, QuestionDialog, FileDialog, ProgressBarDialog
+	ErrorDialog, QuestionDialog, FileDialog, ProgressBarDialog, MessageDialog, \
+	PromptExistingFileDialog, \
+	scrolled_text_view
 from zim.gui.clipboard import Clipboard
-from zim.gui.applications import get_application, get_default_application, CustomToolManager
+from zim.gui.applications import ApplicationManager, CustomToolManager
 
 logger = logging.getLogger('zim.gui')
 
@@ -68,7 +71,7 @@ ui_actions = (
 
 	# name, stock id, label, accelerator, tooltip, readonly
 	('new_page',  'gtk-new', _('_New Page...'), '<ctrl>N', '', False), # T: Menu item
-	('new_sub_page',  'gtk-new', _('New S_ub Page...'), '', '', False), # T: Menu item
+	('new_sub_page',  'gtk-new', _('New S_ub Page...'), '<shift><ctrl>N', '', False), # T: Menu item
 	('open_notebook', 'gtk-open', _('_Open Another Notebook...'), '<ctrl>O', '', True), # T: Menu item
 	('open_new_window', None, _('Open in New _Window'), '', '', True), # T: Menu item
 	('import_page', None, _('_Import Page...'), '', '', False), # T: Menu item
@@ -92,6 +95,7 @@ ui_actions = (
 	('open_document_root', 'gtk-open', _('Open _Document Root'), '', '', True), # T: Menu item
 	('open_document_folder', 'gtk-open', _('Open _Document Folder'), '', '', True), # T: Menu item
 	('attach_file', 'zim-attachment', _('Attach _File'), '', _('Attach external file'), False), # T: Menu item
+	('show_clean_notebook', None, _('_Cleanup Attachments'), '', '', False), # T: Menu item
 	('edit_page_source', 'gtk-edit', _('Edit _Source'), '', '', False), # T: Menu item
 	('show_server_gui', None, _('Start _Web Server'), '', '', True), # T: Menu item
 	('reload_index', None, _('Re-build Index'), '', '', False), # T: Menu item
@@ -258,6 +262,8 @@ class GtkInterface(NotebookInterface):
 	* close-page (page)
 	  Called when closing a page, typically just before a new page is opened
 	  and before closing the application
+	* new-window (window)
+	  Called when a new window is created, can be used as a hook by plugins
 	* preferences-changed
 	  Emitted after the user changed the preferences
 	  (typically triggered by the preferences dialog)
@@ -273,6 +279,7 @@ class GtkInterface(NotebookInterface):
 	__gsignals__ = {
 		'open-page': (gobject.SIGNAL_RUN_LAST, None, (object, object)),
 		'close-page': (gobject.SIGNAL_RUN_LAST, None, (object,)),
+		'new-window': (gobject.SIGNAL_RUN_LAST, None, (object,)),
 		'preferences-changed': (gobject.SIGNAL_RUN_LAST, None, ()),
 		'readonly-changed': (gobject.SIGNAL_RUN_LAST, None, ()),
 		'quit': (gobject.SIGNAL_RUN_LAST, None, ()),
@@ -321,37 +328,22 @@ class GtkInterface(NotebookInterface):
 		if not self.preferences['GtkInterface']['gtk_bell']:
 			gtk.rc_parse_string('gtk-error-bell = 0')
 
-		# Set default applications
-		apps = {
-			'email_client': ['xdg-email', 'startfile'],
-			'file_browser': ['xdg-open', 'startfile'],
-			'web_browser': ['xdg-open', 'startfile']
-		}
-		if ui_environment['platform'] == 'maemo':
-			apps = {
-				'email_client': ['modest'],
-				'file_browser': ['hildon-mime-summon'],
-				'web_browser': ['webbrowser']
-			}
-
-		for type in apps.keys():
-			prefs = self.preferences['GtkInterface']
-			if type in prefs and prefs[type] \
-			and isinstance(prefs[type], basestring):
-				pass # preference is set, no need to set default
-			else:
-				from zim.gui.applications import get_helper_applications
-				key = None
-				helpers = get_helper_applications(type)
-				keys = [entry.key for entry in helpers]
-				for k in apps[type]: # prefered keys
-					if k in keys:
-						key = k
-						break
-				if key is None:
-					if helpers: key = helpers[0].key
-					else: key = 'none'
-				prefs.setdefault(type, key)
+		# Set default applications - check if we already have a default
+		# to prevent unnessecary and relatively slow tryexec() checks
+		manager = ApplicationManager()
+		for type in (
+			'file_browser',
+			'web_browser',
+			'email_client',
+			'text_editor'
+		):
+			if not self.preferences['GtkInterface'].get(type):
+				default = manager.get_default_helper(type)
+				if default:
+					self.preferences['GtkInterface'][type] = default.key
+				else:
+					self.preferences['GtkInterface'][type] = None
+					logger.warn('No helper application defined for %s', type)
 
 		self.mainwindow = MainWindow(self, fullscreen, geometry)
 
@@ -496,6 +488,9 @@ class GtkInterface(NotebookInterface):
 
 		self.check_notebook_needs_upgrade()
 
+		self.save_preferences()
+			# if prefs are modified during init we should save them
+
 		self.mainwindow.show_all()
 		self.mainwindow.pageview.grab_focus()
 		gtk.main()
@@ -593,9 +588,18 @@ class GtkInterface(NotebookInterface):
 			self.uimanager.remove_action_group(handler.actiongroup)
 			handler.actiongroup = None
 
-	@staticmethod
-	def _log_action(action, *a):
-		logger.debug('Action: %s', action.get_name())
+	def _action_handler(self, action, method, *arg):
+		name = action.get_name()
+		logger.debug('Action: %s', name)
+		try:
+			method(*arg)
+		except Exception, error:
+			ErrorDialog(None, error).run()
+			# error dialog also does logging automatically
+
+	def _radio_action_handler(self, object, action, method):
+		# radio action object is not active radio action
+		self._action_handler(action, method, action.get_name())
 
 	def _connect_actions(self, actions, group, handler, is_toggle=False):
 		for name, readonly in [(a[0], a[-1]) for a in actions if not a[0].endswith('_menu')]:
@@ -603,29 +607,29 @@ class GtkInterface(NotebookInterface):
 			action.zim_readonly = readonly
 			if is_toggle: name = 'do_' + name
 			assert hasattr(handler, name), 'No method defined for action %s' % name
-			method = getattr(handler.__class__, name)
-			action.connect('activate', self._log_action)
-			action.connect_object('activate', method, handler)
+			method = getattr(handler, name)
+			action.connect('activate', self._action_handler, method)
 			if self.readonly and not action.zim_readonly:
 				action.set_sensitive(False)
 
 	def add_radio_actions(self, actions, handler, methodname):
 		'''Wrapper for gtk.ActionGroup.add_radio_actions(actions),
 		"handler" is the object that these actions belong to and
-		"methodname" gives the callback to be called on changes in this group.
-		(See doc on gtk.RadioAction 'changed' signal for this callback.)
+		"methodname" gives the callback to be called on changes in this
+		group this method will be called for any change with the name of
+		the active action as only argument.
 		'''
 		# A bit different from the other two methods since radioactions
 		# come in mutual exclusive groups. Only need to connect to one
-		# action to get signals from whole group.
+		# action to get signals from whole group. But need to pass on
+		# the name of the active action
 		assert isinstance(actions[0], tuple), 'BUG: actions should be list of tupels'
 		assert hasattr(handler, methodname), 'No such method %s' % methodname
 		group = self.init_actiongroup(handler)
 		group.add_radio_actions(actions)
-		method = getattr(handler.__class__, methodname)
+		method = getattr(handler, methodname)
 		action = group.get_action(actions[0][0])
-		action.connect('changed', self._log_action)
-		action.connect_object('changed', method, handler)
+		action.connect('changed', self._radio_action_handler, method)
 
 	def add_ui(self, xml, handler):
 		'''Wrapper for gtk.UIManager.add_ui_from_string(xml)'''
@@ -675,7 +679,7 @@ class GtkInterface(NotebookInterface):
 		config file. Each preference is a tuple consisting of:
 
 		* the key in the config file
-		* an option type (see Dialog.add_fields() for more details)
+		* an option type (see InputForm() for more details)
 		* a category (the tab in which the option will be shown)
 		* a label to show in the dialog
 		* a default value
@@ -689,6 +693,18 @@ class GtkInterface(NotebookInterface):
 				register.setdefault(category, [])
 				register[category].append((section, key, type, label))
 
+
+	def register_new_window(self, window):
+		'''Called by windows and dialog to register themselves with
+		the application. Used e.g. by plugins that want to add some
+		widget to specific windows.
+		'''
+		#~ print 'WINDOW:', window
+		self.emit('new-window', window)
+
+	def do_new_window(self, window):
+		pass # TODO: keep register of pageviews
+
 	def get_path_context(self):
 		'''Returns the current 'context' for actions that want a path to start
 		with. Asks the mainwindow for a selected page, defaults to the
@@ -719,8 +735,15 @@ class GtkInterface(NotebookInterface):
 			# We are already intialized, so let another process handle it
 			if self.usedaemon:
 				from zim.daemon import DaemonProxy
+				if isinstance(notebook, basestring) \
+				and notebook.startswith('zim+') \
+				and '?' in notebook:
+					# Interwiki link with page name attached
+					notebook, pagename = notebook.split('?', 1)
+				else:
+					pagename = None
 				notebook = DaemonProxy().get_notebook(notebook)
-				notebook.present()
+				notebook.present(page=pagename)
 			else:
 				self.spawn(notebook)
 
@@ -797,7 +820,7 @@ class GtkInterface(NotebookInterface):
 	def open_page(self, path=None):
 		'''Emit the open-page signal. The argument 'path' can either be a Page
 		or a Path object. If 'page' is None a dialog is shown
-		to specify the page. If 'path' is a HistoryRecord we assume that this
+		to specify the page. If 'path' is a HistoryPath we assume that this
 		call is the result of a history action and the page is not added to
 		the history. The original path object is given as the second argument
 		in the signal, so handlers can inspect how this method was called.
@@ -835,7 +858,7 @@ class GtkInterface(NotebookInterface):
 		parent = self.actiongroup.get_action('open_page_parent')
 		child = self.actiongroup.get_action('open_page_child')
 
-		if isinstance(path, HistoryRecord):
+		if isinstance(path, HistoryPath):
 			historyrecord = path
 			self.history.set_current(path)
 			back.set_sensitive(not path.is_first)
@@ -934,36 +957,47 @@ class GtkInterface(NotebookInterface):
 		is expected behavior for users not yet fully aware of the automatic
 		create/save/delete behavior in zim.
 		'''
-		NewPageDialog(self).run()
+		NewPageDialog(self, path=self.get_path_context()).run()
 
 	def new_sub_page(self):
 		'''Same as new_page() but sets the namespace widget one level deeper'''
 		NewPageDialog(self, path=self.get_path_context(), subpage=True).run()
 
-	def new_page_from_text(self, text, name=None):
+	def new_page_from_text(self, text, name=None, open_page=False):
 		'''Create a new page and set text directly. If no name is given
 		the first line of the text is used as basename. If the page
 		already exists a number is added to force a unique page name.
 		'''
+		# The 'open_page' argument is a bit of a hack for remote calls
+		# it is needed because the remote function doesn't know the
+		# exact page name we creates...
 		if not name:
 			name = text.strip()[:30]
 			if '\n' in name:
 				name, _ = name.split('\n', 1)
-			name = self.notebook.cleanup_pathname(
-				name.replace(':', ''), purge=True)
+			name = self.notebook.cleanup_pathname(name.replace(':', ''), purge=True)
+		elif isinstance(name, Path):
+			name = name.name
+			name = self.notebook.cleanup_pathname(name, purge=True)
+		else:
+			name = self.notebook.cleanup_pathname(name, purge=True)
+
 		path = self.notebook.resolve_path(name)
 		page = self.notebook.get_new_page(path)
-		page.parse('plain', text)
+		page.parse('wiki', text) # FIXME format hard coded
 		self.notebook.store_page(page)
-		self.open_page(page)
+		if open_page:
+			self.open_page(page)
+		return page
 
 	def append_text_to_page(self, name, text):
 		'''Append text to an (exising) page'''
+		if isinstance(name, Path):
+			name = name.name
 		path = self.notebook.resolve_path(name)
 		page = self.notebook.get_page(path)
-		page.parse('plain', text, append=True)
+		page.parse('wiki', text, append=True) # FIXME format hard coded
 		self.notebook.store_page(page)
-		self.open_page(page)
 
 	def open_new_window(self, page=None):
 		'''Open page in a new window'''
@@ -1156,8 +1190,35 @@ class GtkInterface(NotebookInterface):
 		self.open_page(self.notebook.get_page(self.page))
 
 	def attach_file(self, path=None):
+		'''Show the AttachFileDialog'''
 		AttachFileDialog(self, path=path).run()
 
+	def do_attach_file(self, path, file, force_overwrite=False):
+		'''Callback for AttachFileDialog and InsertImageDialog
+		When 'force_overwrite' is False the user will be prompted in
+		case the new file has the same name as an existing attachment.
+		Returns the (new) filename or None when the action was canceled.
+		'''
+		namechanged = False
+		dir = self.notebook.get_attachments_dir(path)
+		if dir is None:
+			raise Error, '%s does not have an attachments dir' % path
+
+		dest = dir.file(file.basename)
+		if dest.exists() and not force_overwrite:
+			dialog = PromptExistingFileDialog(self, dest)
+			dest = dialog.run()
+			if dest is None:
+				return None	# dialog was cancelled
+
+		file.copyto(dest)
+		return dest
+
+	def show_clean_notebook(self):
+		'''Show the CleanNotebookDialog'''
+		from zim.gui.cleannotebookdialog import CleanNotebookDialog
+		CleanNotebookDialog(self).run()
+
 	def open_file(self, file):
 		'''Open either a File or a Dir in the file browser'''
 		assert isinstance(file, (File, Dir))
@@ -1167,27 +1228,44 @@ class GtkInterface(NotebookInterface):
 		if file.exists():
 			# TODO if isinstance(File) check default application for mime type
 			# this is needed once we can set default app from "open with.." menu
-			self._openwith(
-				self.preferences['GtkInterface']['file_browser'], (file,) )
+			self.open_with('file_browser', file)
 		else:
-			ErrorDialog(self, NoSuchFileError(file)).run()
+			raise NoSuchFileError, file
 
 	def open_url(self, url):
 		assert isinstance(url, basestring)
 		if url.startswith('file:/'):
 			self.open_file(File(url))
 		elif url.startswith('mailto:'):
-			self._openwith(self.preferences['GtkInterface']['email_client'], (url,))
+			self.open_with('email_client', url)
 		elif url.startswith('zim+'):
 			self.open_notebook(url)
+		elif url.startswith('outlook:') and hasattr(os, 'startfile'):
+			# Special case for outlook folder paths on windows
+			os.startfile(url)
 		else:
 			if is_win32_share_re.match(url):
 				url = normalize_win32_share(url)
-			self._openwith(self.preferences['GtkInterface']['web_browser'], (url,))
+			self.open_with('web_browser', url)
 
-	def _openwith(self, name, args):
-		entry = get_application(name)
-		entry.spawn(args)
+	def open_with(self, app_type, uri):
+		'''Open an uri or an url with a specific app type. Type can be
+		'file_browser', 'web_browser' or 'email_client'.
+
+		NOTE: only use this method when you need to force the app type,
+		otherwise use either open_file() or open_url().
+		'''
+		def check_error(status):
+			if status != 0:
+					ErrorDialog(self, _('Could not open: %s') % uri).run()
+					# T: error when external application fails
+
+		app = self.preferences['GtkInterface'][app_type]
+		entry = ApplicationManager().get_application(app)
+		try:
+			entry.spawn((uri,), callback=check_error)
+		except NotImplementedError:
+			entry.spawn((uri,)) # E.g. webbrowser module
 
 	def open_attachments_folder(self):
 		dir = self.notebook.get_attachments_dir(self.page)
@@ -1259,9 +1337,9 @@ class GtkInterface(NotebookInterface):
 			ErrorDialog('This page does not have a source file').run()
 			return
 
-		if self._edit_file(file):
-			if page == self.page:
-				self.reload_page()
+		self.edit_file(file, istextfile=True)
+		if page == self.page:
+			self.reload_page()
 
 	def edit_config_file(self, configfile):
 		if not configfile.file.exists():
@@ -1269,21 +1347,48 @@ class GtkInterface(NotebookInterface):
 				configfile.default.copyto(configfile.file)
 			else:
 				configfile.file.touch()
-		self._edit_file(configfile.file)
+		self.edit_file(configfile.file, istextfile=True)
 
-	def _edit_file(self, file):
-		# We hope the application we get here does not return before
-		# editing is done, but unfortunately we have no way to force
-		# this. So may e.g. open a new tab in an existing window and
-		# return immediatly.
-		application = get_default_application('text/plain')
-		try:
-			application.run((file,))
-		except:
-			logger.exception('Error while running %s:', application.name)
-			return False
-		else:
-			return True
+	def edit_file(self, file, istextfile=None):
+		'''Edit a file with and external application and wait. Spawns a dialog to block the zim gui
+		while the axternal application is running. Dialog is closed automatically when the application
+		exits after modifying the file. If the file is unmodified the user needs to click the "Done"
+		button in the dialog because we can not know if the application was really done or just forked.
+
+		If 'istextfile' is True the text editor from the preferences menu is used, if it is False the
+		file browser is used and if it is None we check the mimetype.
+		'''
+		if not file.exists():
+			raise NoSuchFileError, file
+
+		oldmtime = file.mtime()
+
+		dialog = MessageDialog(self, (
+			_('Editing file: %s') % file.basename,
+				# T: main text for dialog for editing external files
+			_('You are editing a file in an external application. You can close this dialog when you are done')
+				# T: description for dialog for editing external files
+		) )
+
+		def check_close_dialog(status):
+			if status != 0:
+				dialog.destroy()
+				ErrorDialog(self, _('Could not open: %s') % uri).run()
+					# T: error when external application fails
+			else:
+				newmtime = file.mtime()
+				if newmtime != oldmtime:
+					dialog.destroy()
+
+		if istextfile is None:
+			istextfile = file.get_mimetype().startswith('text/')
+
+		if istextfile: app = 'text_editor'
+		else:          app = 'file_browser'
+
+		entry = ApplicationManager().get_application(self.preferences['GtkInterface'][app])
+		entry.spawn((file,), callback=check_close_dialog)
+		dialog.run()
 
 	def show_server_gui(self):
 		# TODO instead of spawn, include in this process
@@ -1402,7 +1507,7 @@ class GtkInterface(NotebookInterface):
 			else:
 				tool.run(args)
 				self.reload_page()
-				#~ self.notebook.index.update(background=True)
+				self.notebook.index.update(background=True)
 				# TODO instead of using run, use spawn and show dialog
 				# with cancel button. Dialog blocks ui.
 		except Exception, error:
@@ -1479,23 +1584,18 @@ class MainWindow(Window):
 			return True # Do not destroy - let close() handle it
 		self.connect('delete-event', do_delete_event)
 
-		vbox = gtk.VBox()
-		self.add(vbox)
+		# setup the window layout
+		from zim.gui.widgets import TOP, BOTTOM, TOP_PANE, LEFT_PANE
 
 		# setup menubar and toolbar
 		self.add_accel_group(ui.uimanager.get_accel_group())
 		self.menubar = ui.uimanager.get_widget('/menubar')
 		self.toolbar = ui.uimanager.get_widget('/toolbar')
 		self.toolbar.connect('popup-context-menu', self.do_toolbar_popup)
-		vbox.pack_start(self.menubar, False)
-		vbox.pack_start(self.toolbar, False)
+		self.add_bar(self.menubar, TOP)
+		self.add_bar(self.toolbar, TOP)
 
-		# split window in side pane and editor
-		self.hpane = gtk.HPaned()
-		self.hpane.set_position(175)
-		vbox.add(self.hpane)
-		self.sidepane = gtk.VBox(spacing=5)
-		self.hpane.add1(self.sidepane)
+		self.sidepane = self._zim_window_left # FIXME - get rid of sidepane attribute
 
 		self.sidepane.connect('key-press-event',
 			lambda o, event: event.keyval == KEYVAL_ESC
@@ -1503,24 +1603,29 @@ class MainWindow(Window):
 
 
 		self.pageindex = PageIndex(ui)
-		self.sidepane.add(self.pageindex)
+		self.add_tab(_('Index'), self.pageindex, LEFT_PANE) # T: Label for pageindex tab
 
-		vbox2 = gtk.VBox()
-		self.hpane.add2(vbox2)
+		def check_focus_sidepane(window, widget):
+			focus = widget == self.pageindex
+				# FIXME - what if we have more widgets in side pane ?
+			if not focus:
+				self.on_sidepane_lost_focus()
+
+		self.connect('set-focus', check_focus_sidepane)
 
 		self.pathbar = None
 		self.pathbar_box = gtk.HBox() # FIXME other class for this ?
 		self.pathbar_box.set_border_width(3)
-		vbox2.pack_start(self.pathbar_box, False)
+		self.add_widget(self.pathbar_box, TOP_PANE, TOP)
 
 		self.pageview = PageView(ui)
 		self.pageview.view.connect_after(
 			'toggle-overwrite', self.do_textview_toggle_overwrite)
-		vbox2.add(self.pageview)
+		self.add(self.pageview)
 
 		# create statusbar
 		hbox = gtk.HBox(spacing=0)
-		vbox.pack_start(hbox, False, True, False)
+		self.add_bar(hbox, BOTTOM)
 
 		self.statusbar = gtk.Statusbar()
 		if ui_environment['platform'] == 'maemo':
@@ -1608,12 +1713,12 @@ class MainWindow(Window):
 		group = gtk.AccelGroup()
 		group.connect_group( # <Alt><Space>
 			space, gtk.gdk.MOD1_MASK, gtk.ACCEL_VISIBLE,
-			self.do_switch_focus)
+			self.toggle_focus_sidepane)
 
 		if self.ui.preferences['GtkInterface']['toggle_on_ctrlspace']:
 			group.connect_group( # <Ctrl><Space>
 				space, gtk.gdk.CONTROL_MASK, gtk.ACCEL_VISIBLE,
-				self.do_switch_focus)
+				self.toggle_focus_sidepane)
 
 		self.add_accel_group(group)
 		self._switch_focus_accelgroup = group
@@ -1739,10 +1844,10 @@ class MainWindow(Window):
 		if show:
 			self.sidepane.set_no_show_all(False)
 			self.sidepane.show_all()
-			self.hpane.set_position(self.uistate['sidepane_pos'])
+			self._zim_window_left_pane.set_position(self.uistate['sidepane_pos'])
 			self.pageindex.grab_focus()
 		else:
-			self.uistate['sidepane_pos'] = self.hpane.get_position()
+			self.uistate['sidepane_pos'] = self._zim_window_left_pane.get_position()
 			self.sidepane.hide_all()
 			self.sidepane.set_no_show_all(True)
 			self.pageview.grab_focus()
@@ -1750,7 +1855,11 @@ class MainWindow(Window):
 		self._sidepane_autoclose = False
 		self.uistate['show_sidepane'] = show
 
-	def do_switch_focus(self, *a):
+	def toggle_focus_sidepane(self, *a):
+		'''Switch focus between the textview and the sidepane.
+		Automatically opens the sidepane if it is closed
+		(but sets a property to automatically close it again).
+		'''
 		action = self.actiongroup.get_action('toggle_sidepane')
 		if action.get_active():
 			# side pane open
@@ -1769,6 +1878,12 @@ class MainWindow(Window):
 
 		return True # we are called from an event handler
 
+	def on_sidepane_lost_focus(self):
+		action = self.actiongroup.get_action('toggle_sidepane')
+		if self._sidepane_autoclose and action.get_active():
+			# Sidepane open and should close automatic
+			self.toggle_sidepane(show=False)
+
 	def set_pathbar(self, style):
 		'''Set the pathbar. Style can be either PATHBAR_NONE,
 		PATHBAR_RECENT, PATHBAR_HISTORY or PATHBAR_PATH.
@@ -1776,8 +1891,7 @@ class MainWindow(Window):
 		assert style in ('none', 'recent', 'history', 'path')
 		self.actiongroup.get_action('set_pathbar_'+style).activate()
 
-	def do_set_pathbar(self, action):
-		name = action.get_name()
+	def do_set_pathbar(self, name):
 		style = name[12:] # len('set_pathbar_') == 12
 
 		if style == PATHBAR_NONE:
@@ -1813,8 +1927,7 @@ class MainWindow(Window):
 		assert style in ('icons_and_text', 'icons_only', 'text_only'), style
 		self.actiongroup.get_action('set_toolbar_'+style).activate()
 
-	def do_set_toolbar_style(self, action):
-		name = action.get_name()
+	def do_set_toolbar_style(self, name):
 		style = name[12:] # len('set_toolbar_') == 12
 
 		if style == TOOLBAR_ICONS_AND_TEXT:
@@ -1835,8 +1948,7 @@ class MainWindow(Window):
 		assert size in ('large', 'small', 'tiny'), size
 		self.actiongroup.get_action('set_toolbar_icons_'+size).activate()
 
-	def do_set_toolbar_size(self, action):
-		name = action.get_name()
+	def do_set_toolbar_size(self, name):
 		size = name[18:] # len('set_toolbar_icons_') == 18
 
 		if size == TOOLBAR_ICONS_LARGE:
@@ -1874,7 +1986,7 @@ class MainWindow(Window):
 
 		if not self._geometry_set:
 			# Ignore this is a explicit geometry was specified to the constructor
-			self.uistate.setdefault('windowsize', (600, 450), check=self.uistate.is_coord)
+			self.uistate.setdefault('windowsize', (600, 450), check=value_is_coord)
 			w, h = self.uistate['windowsize']
 			self.set_default_size(w, h)
 
@@ -1967,7 +2079,7 @@ class MainWindow(Window):
 		w, h = self.get_size()
 		if not self._fullscreen:
 			self.uistate['windowsize'] = (w, h)
-		self.uistate['sidepane_pos'] = self.hpane.get_position()
+		self.uistate['sidepane_pos'] = self._zim_window_left_pane.get_position()
 
 	def do_textview_toggle_overwrite(self, view):
 		state = view.get_overwrite()
@@ -2025,7 +2137,7 @@ class PageWindow(Window):
 		self.uistate = ui.uistate['PageWindow']
 			# TODO remember for separate windows separately
 			# e.g. use PageWindow1, PageWindow2, etc
-		self.uistate.setdefault('windowsize', (500, 400), check=self.uistate.is_coord)
+		self.uistate.setdefault('windowsize', (500, 400), check=value_is_coord)
 		w, h = self.uistate['windowsize']
 		self.set_default_size(w, h)
 
@@ -2131,21 +2243,22 @@ discarded, but you can restore the copy later.''')
 		ErrorDialog.run(self)
 		gobject.source_remove(id)
 
-
 class OpenPageDialog(Dialog):
 	'''Dialog to go to a specific page. Also known as the "Jump to" dialog.
 	Prompts for a page name and navigate to that page on 'Ok'.
 	'''
 
-	def __init__(self, ui, namespace=None):
+	def __init__(self, ui):
 		Dialog.__init__(self, ui, _('Jump to'), # T: Dialog title
 			button=(None, gtk.STOCK_JUMP_TO),
-			path_context = ui.page,
-			fields=[('name', 'page', _('Jump to Page'), None)] # T: Label for page input
+		)
+
+		self.add_form(
+			[('page', 'page', _('Jump to Page'), ui.page)] # T: Label for page input
 		)
 
 	def do_response_ok(self):
-		path = self.get_field('name')
+		path = self.form['page']
 		if path:
 			self.ui.open_page(path)
 			return True
@@ -2168,34 +2281,45 @@ class NewPageDialog(Dialog):
 				'Please note that linking to a non-existing page\n'
 				'also creates a new page automatically.'),
 			# T: Dialog text in 'new page' dialog
-			path_context=path,
-			fields=[('name', 'page', _('Page Name'), None)], # T: Input label
 			help=':Help:Pages'
 		)
 
+		self.path = path or ui.page
+
+		templates = list_templates('wiki')
+		self.add_form([
+			('page', 'page', _('Page Name'), (path or ui.page)), # T: Input label
+			('template', 'choice', _('Page Template'), templates) # T: Choice label
+		], None, None, False )
+
+		key = self.path or ''
+		default = ui.notebook.namespace_properties[key]['template']
+		self.form['template'] = default
+		self.form.widgets['template'].set_no_show_all(True) # TEMP: hide feature
+		self.form.widgets['template'].set_property('visible', False) # TEMP: hide feature
+
 		if subpage:
-			print 'SETTING force_child'
-			pageentry = self.inputs['name']
-			pageentry.force_child = True
+			self.form.widgets['page'].force_child = True
+
+		# TODO: reset default when page input changed
 
 	def do_response_ok(self):
-		path = self.get_field('name')
-		if path:
-			page = self.ui.notebook.get_page(path)
-			if page.hascontent or page.haschildren:
-				ErrorDialog(self, _('Page exists')+': %s' % page.name).run() # T: error message
-				return False
-
-			template = self.ui.notebook.get_template(page)
-			tree = template.process_to_parsetree(self.ui.notebook, page)
-			page.set_parsetree(tree)
-			self.ui.save_page()
-
-			self.ui.open_page(page)
-			return True
-		else:
+		path = self.form['page']
+		if not path:
 			return False
 
+		page = self.ui.notebook.get_page(path)
+		if page.hascontent or page.haschildren:
+			raise Error, _('Page exists')+': %s' % page.name
+				# T: Error when creating new page
+
+		template = get_template('wiki', self.form['template'])
+		tree = template.process_to_parsetree(self.ui.notebook, page)
+		page.set_parsetree(tree)
+		self.ui.open_page(page)
+		self.ui.save_page() # Save new page directly
+		return True
+
 
 class SaveCopyDialog(FileDialog):
 
@@ -2265,34 +2389,41 @@ class MovePageDialog(Dialog):
 			raise AssertionError, 'Could not save page'
 			# assert statement could be optimized away
 
-		i = self.ui.notebook.index.n_list_links_to_tree(
-					self.path, zim.index.LINK_DIR_BACKWARD )
-
 		self.vbox.add(gtk.Label(_('Move page "%s"') % self.path.name))
 			# T: Heading in 'move page' dialog - %s is the page name
-		linkslabel = ngettext(
+
+		indexpath = self.ui.notebook.index.lookup_path(self.path)
+		if indexpath:
+			i = self.ui.notebook.index.n_list_links_to_tree(
+					indexpath, zim.index.LINK_DIR_BACKWARD )
+		else:
+			i = 0
+
+		label = ngettext(
 			'Update %i page linking to this page',
 			'Update %i pages linking to this page', i) % i
 			# T: label in MovePage dialog - %i is number of backlinks
 			# TODO update lable to reflect that links can also be to child pages
 		self.context_page = self.path.parent
-		self.add_fields([
+		self.add_form([
 			('parent', 'namespace', _('Namespace'), self.context_page),
 				# T: Input label for namespace to move a file to
-			('links', 'bool', linkslabel, True),
+			('update', 'bool', label),
 				# T: option in 'move page' dialog
 		])
 
 		if i == 0:
-			self.inputs['links'].set_active(False)
-			self.inputs['links'].set_sensitive(False)
+			self.form['update'] = False
+			self.form.widgets['update'].set_sensitive(False)
+		else:
+			self.form['update'] = True
 
 	def do_response_ok(self):
-		parent = self.get_field('parent')
-		links = self.get_field('links')
+		parent = self.form['parent']
+		update = self.form['update']
 		newpath = parent + self.path.basename
 		self.hide() # hide this dialog before showing the progressbar
-		ok = self.ui.do_move_page(self.path, newpath, update_links=links)
+		ok = self.ui.do_move_page(self.path, newpath, update)
 		if ok:
 			return True
 		else:
@@ -2313,40 +2444,49 @@ class RenamePageDialog(Dialog):
 		page = self.ui.notebook.get_page(self.path)
 		existing = (page.hascontent or page.haschildren)
 
-		i = self.ui.notebook.index.n_list_links_to_tree(
-					self.path, zim.index.LINK_DIR_BACKWARD )
-
 		self.vbox.add(gtk.Label(_('Rename page "%s"') % self.path.name))
 			# T: label in 'rename page' dialog - %s is the page name
-		linkslabel = ngettext(
+
+		indexpath = self.ui.notebook.index.lookup_path(self.path)
+		if indexpath:
+			i = self.ui.notebook.index.n_list_links_to_tree(
+					indexpath, zim.index.LINK_DIR_BACKWARD )
+		else:
+			i = 0
+
+		label = ngettext(
 			'Update %i page linking to this page',
 			'Update %i pages linking to this page', i) % i
 			# T: label in MovePage dialog - %i is number of backlinks
-			# TODO update lable to reflect that links can also be to child pages
-		self.add_fields([
-			('name', 'string', _('Name'), self.path.basename),
+			# TODO update label to reflect that links can also be to child pages
+
+		self.add_form([
+			('name', 'string', _('Name')),
 				# T: Input label in the 'rename page' dialog for the new name
-			('head', 'bool', _('Update the heading of this page'), existing),
+			('head', 'bool', _('Update the heading of this page')),
 				# T: Option in the 'rename page' dialog
-			('links', 'bool', linkslabel, True),
+			('update', 'bool', label),
 				# T: Option in the 'rename page' dialog
-		])
+		], {
+			'name': self.path.basename,
+			'head': existing,
+			'update': True,
+		})
 
 		if not existing:
-			self.inputs['head'].set_active(False)
-			self.inputs['head'].set_sensitive(False)
+			self.form['head'] = False
+			self.form.widgets['head'].set_sensitive(False)
 
 		if i == 0:
-			self.inputs['links'].set_active(False)
-			self.inputs['links'].set_sensitive(False)
+			self.form['update'] = False
+			self.form.widgets['update'].set_sensitive(False)
 
 	def do_response_ok(self):
-		name = self.get_field('name')
-		head = self.get_field('head')
-		links = self.get_field('links')
+		name = self.form['name']
+		head = self.form['head']
+		update = self.form['update']
 		self.hide() # hide this dialog before showing the progressbar
-		ok = self.ui.do_rename_page(
-			self.path, newbasename=name, update_heading=head, update_links=links)
+		ok = self.ui.do_rename_page(self.path, name, head, update)
 		if ok:
 			return True
 		else:
@@ -2371,7 +2511,7 @@ class DeletePageDialog(Dialog):
 		hbox.pack_start(img, False)
 
 		vbox = gtk.VBox(spacing=5)
-		hbox.add(vbox)
+		hbox.pack_start(vbox, False)
 
 		label = gtk.Label()
 		short = _('Delete page "%s"?') % self.path.basename
@@ -2381,14 +2521,19 @@ class DeletePageDialog(Dialog):
 		label.set_markup('<b>'+short+'</b>\n\n'+long)
 		vbox.pack_start(label, False)
 
-		i = self.ui.notebook.index.n_list_links_to_tree(
-					self.path, zim.index.LINK_DIR_BACKWARD )
-		linkslabel = ngettext(
+		indexpath = self.ui.notebook.index.lookup_path(self.path)
+		if indexpath:
+			i = self.ui.notebook.index.n_list_links_to_tree(
+					indexpath, zim.index.LINK_DIR_BACKWARD )
+		else:
+			i = 0
+
+		label = ngettext(
 			'Remove links from %i page linking to this page',
 			'Remove links from %i pages linking to this page', i) % i
 			# T: label in DeletePage dialog - %i is number of backlinks
 			# TODO update lable to reflect that links can also be to child pages
-		self.links_checkbox = gtk.CheckButton(label=linkslabel)
+		self.links_checkbox = gtk.CheckButton(label=label)
 		vbox.pack_start(self.links_checkbox, False)
 
 		if i == 0:
@@ -2397,6 +2542,24 @@ class DeletePageDialog(Dialog):
 		else:
 			self.links_checkbox.set_active(True)
 
+
+		# TODO use expander here
+		dir = self.ui.notebook.get_attachments_dir(self.path)
+		text = dir.get_file_tree_as_text(raw=True)
+		n = len([l for l in text.splitlines() if not l.endswith('/')])
+
+		string = ngettext('%i file will be deleted', '%i files will be deleted', n) % n
+			# T: label in the DeletePage dialog to warn user of attachments being deleted
+		if n > 0:
+			string = '<b>'+string+'</b>'
+
+		label = gtk.Label()
+		label.set_markup('\n'+string+':')
+		self.vbox.add(label)
+		window, textview = scrolled_text_view(text, monospace=True)
+		window.set_size_request(250, 200)
+		self.vbox.add(window)
+
 	def do_response_ok(self):
 		update_links = self.links_checkbox.get_active()
 
@@ -2408,9 +2571,8 @@ class DeletePageDialog(Dialog):
 		try:
 			self.ui.notebook.delete_page(self.path, update_links, callback)
 		except Exception, error:
-			ErrorDialog(self, error).run()
 			dialog.destroy()
-			return False
+			raise
 		else:
 			dialog.destroy()
 			return True
@@ -2428,8 +2590,8 @@ class AttachFileDialog(FileDialog):
 			self.path = path
 		assert self.path, 'Need a page here'
 
-		self.dir = self.ui.notebook.get_attachments_dir(self.path)
-		if self.dir is None:
+		dir = self.ui.notebook.get_attachments_dir(self.path)
+		if dir is None:
 			ErrorDialog(_('Page "%s" does not have a folder for attachments') % self.path)
 				# T: Error dialog - %s is the full page name
 			raise Exception, 'Page "%s" does not have a folder for attachments' % self.path
@@ -2451,15 +2613,16 @@ class AttachFileDialog(FileDialog):
 			# Similar code in zim.gui.InsertImageDialog
 
 		for file in files:
-			file.copyto(self.dir)
-			file = self.dir.file(file.basename)
+			file = self.ui.do_attach_file(self.path, file)
+			if file is None:
+				return False # Cancelled overwrite dialog
+
 			pageview = self.ui.mainwindow.pageview
 			if self.uistate['insert_attached_images'] and file.isimage():
-				try:
-					pageview.insert_image(file, interactive=False)
-				except:
-					logger.exception('Could not insert image')
-					pageview.insert_links([file]) # image type not supported?
+				ok = pageview.insert_image(file, interactive=False)
+				if not ok: # image type not supported?
+					logger.info('Could not insert image: %s', file)
+					pageview.insert_links([file])
 			else:
 				pageview.insert_links([file])
 
