@@ -6,8 +6,8 @@ import gtk
 import logging
 
 from zim.fs import *
-from zim.gui.widgets import Dialog, ImageView, Button, QuestionDialog, scrolled_text_view
-
+from zim.gui.widgets import ui_environment, \
+	Dialog, ImageView, Button, QuestionDialog, scrolled_text_view
 
 logger = logging.getLogger('zim.gui')
 
@@ -22,27 +22,36 @@ class ImageGeneratorDialog(Dialog):
 	# TODO: use uistate to remember pane position
 
 	def __init__(self, ui, title, generator, image=None, **opt):
-		Dialog.__init__(self, ui, title, defaultwindowsize=(450, 300), **opt)
+		if ui_environment['platform'] == 'maemo':
+			defaultsize = (450,480)
+			# Use maximum available vertical space because decorations take
+			# too much real state
+		else:
+			defaultsize = (450,300)
+		Dialog.__init__(self, ui, title, defaultwindowsize=defaultsize, **opt)
+		if ui_environment['platform'] == 'maemo':
+			self.resize(450,480)
+			# Force maximum dialog size under maemo, otherwise
+			# we'll end with a too small dialog and no way to resize it
 		self.generator = generator
 		self.imagefile = None
 		self.logfile = None
 		self._existing_file = None
 
 		self.vpane = gtk.VPaned()
-		self.vpane.set_position(100)
+		self.vpane.set_position(150)
 		self.vbox.add(self.vpane)
 
 		self.imageview = ImageView(bgcolor='#FFF', checkboard=False)
-		# TODO scrolled window
 		self.vpane.add1(self.imageview)
+		# TODO scrolled window and option to zoom in / real size
 
-		self.textview = gtk.TextView()
-		# TODO scrolled window
+		window, textview = scrolled_text_view(monospace=True)
+		self.textview = textview
+		self.textview.set_editable(True)
+		self.vpane.add2(window)
 		# TODO supply at least an Undo stack for this textview
 		# or optionally subclass from gtksourceview
-		self.textview.set_left_margin(5)
-		self.textview.set_right_margin(5)
-		self.vpane.add2(self.textview)
 
 		hbox = gtk.HBox(spacing=5)
 		self.vbox.pack_start(hbox, False)
