@@ -7,9 +7,8 @@ import gtk
 import logging
 
 import zim.plugins
-from zim.gui.applications import \
-	get_application, get_helper_applications, CustomCommandDialog
-from zim.gui.widgets import Dialog, Button, BrowserTreeView, scrolled_text_view
+from zim.gui.applications import ApplicationManager, NewApplicationDialog
+from zim.gui.widgets import Dialog, Button, BrowserTreeView, scrolled_text_view, InputForm, input_table_factory
 from zim.gui.pageview import PageView
 
 
@@ -22,29 +21,36 @@ class PreferencesDialog(Dialog):
 	registered using GtkInterface.register_preferences().
 	'''
 
+	OTHER_APP = _('Other Application') + '...'
+		# T: label to pop dialog with more applications in 'open with' menu
+
 	def __init__(self, ui):
 		Dialog.__init__(self, ui, _('Preferences')) # T: Dialog title
 		gtknotebook = gtk.Notebook()
 		self.vbox.add(gtknotebook)
 
 		# Dynamic tabs
+		self.forms = {}
 		for category, preferences in ui.preferences_register.items():
-			table = gtk.Table()
-			table.set_border_width(5)
-			table.set_row_spacings(12)
-			table.set_col_spacings(12)
 			vbox = gtk.VBox()
-			vbox.pack_start(table, False)
 			gtknotebook.append_page(vbox, gtk.Label(category))
+
 			fields = []
+			values = {}
+			sections = {}
 			for p in preferences:
 				section, key, type, label = p
-				value = ui.preferences[section][key]
-				fields.append(((section, key), type, label, value))
-				# a tuple is hashable and can be used as field name...
-			self.add_fields(fields, table=table, trigger_response=False)
+				fields.append((key, type, label))
+				values[key] = ui.preferences[section][key]
+				sections[key] = section
+
+			form = InputForm(fields, values)
+			form.preferences_sections = sections
+			vbox.pack_start(form, False)
+			self.forms[category] = form
+
 			if category == 'Interface':
-				self._add_font_selection(table, vbox)
+				self._add_font_selection(form)
 
 		# Styles tab
 		#~ gtknotebook.append_page(StylesTab(self), gtk.Label('Styles'))
@@ -53,64 +59,64 @@ class PreferencesDialog(Dialog):
 		#~ gtknotebook.append_page(KeyBindingsTab(self), gtk.Label('Key bindings'))
 
 		# Applications tab
-		table = gtk.Table()
-		table.set_border_width(5)
-		table.set_row_spacings(12)
-		table.set_col_spacings(12)
-		self.add_fields( (
-			('file_browser', 'list', _('File browser'), (None, ())),
-				# T: Input for application type in preferences dialog
-			('web_browser', 'list', _('Web browser'), (None, ())),
-				# T: Input for application type in preferences dialog
-			('email_client', 'list', _('Email client'), (None, ())),
-				# T: Input for application type in preferences dialog
-		), table=table, trigger_response=False)
-		for type in ('file_browser', 'web_browser', 'email_client'):
-			self._append_applications(type)
 		vbox = gtk.VBox()
-		vbox.pack_start(table, False)
 		gtknotebook.append_page(vbox, gtk.Label(_('Applications')))
 				# T: Heading in preferences dialog
 
+		form = InputForm( (
+			('file_browser', 'choice', _('File browser'), ()),
+				# T: Input for application type in preferences dialog
+			('web_browser', 'choice', _('Web browser'), ()),
+				# T: Input for application type in preferences dialog
+			('email_client', 'choice', _('Email client'), ()),
+				# T: Input for application type in preferences dialog
+			('text_editor', 'choice', _('Text Editor'), ()),
+				# T: Input for application type in preferences dialog
+		) )
+		for type, widget in form.widgets.items():
+			self._append_applications(type, widget)
+
+		vbox.pack_start(form, False)
+		self.applicationsform = form
+
 		# Plugins tab
 		gtknotebook.append_page(PluginsTab(self), gtk.Label(_('Plugins')))
 				# T: Heading in preferences dialog
 
-	def _add_font_selection(self, table, vbox):
+	def _add_font_selection(self, table):
 		# need to hardcode this, can not register it as a preference
-		self.add_fields((
-			('use_custom_font', 'bool', _('Use a custom font'), False),),
-				# T: Option in preferences dialog
-			table=table, trigger_response=False)
-		self.use_custom_font = self.inputs.pop('use_custom_font')
+		table.add_inputs( (
+			('use_custom_font', 'bool', _('Use a custom font')), 
+			# T: option in preferences dialog
+		) )
+		table.preferences_sections['use_custom_font'] = 'GtkInterface'
 
 		self.fontbutton = gtk.FontButton()
+		self.fontbutton.set_use_font(True) # preview in button
 		self.fontbutton.set_sensitive(False)
 		try:
 			font = PageView.style['TextView']['font']
 			if font:
 				self.fontbutton.set_font_name(font)
 				self.fontbutton.set_sensitive(True)
-				self.use_custom_font.set_active(True)
+				table['use_custom_font'] = True
 		except KeyError:
 			pass
-		self.use_custom_font.connect('toggled',
-			lambda o: self.fontbutton.set_sensitive(self.use_custom_font.get_active()))
 
-		# HACK - how to do proper layout fontbutton ?
-		self.fontbutton.set_size_request(200, -1)
-		hbox = gtk.HBox()
-		vbox.pack_start(hbox, False)
-		hbox.pack_start(gtk.Label('\t\t'), False)
-		hbox.pack_start(self.fontbutton, False)
+		table.widgets['use_custom_font'].connect('toggled',
+			lambda o: self.fontbutton.set_sensitive(o.get_active()) )
 
-	def _append_applications(self, type):
-		# TODO search for other options
+		self.fontbutton.set_size_request(100, -1)
+		input_table_factory(((None, self.fontbutton),), table)
+
+	def _append_applications(self, type, widget):
+		manager = ApplicationManager()
 
 		current = self.ui.preferences['GtkInterface'][type]
-		apps = get_helper_applications(type)
-		if not current in [app.key for app in apps]:
-			app = get_application(current)
+		apps = manager.list_helpers(type)
+		if not current is None \
+		and not current in [app.key for app in apps]:
+			app = manager.get_application(current)
 			if app:
 				apps.insert(0, app)
 			else:
@@ -119,37 +125,61 @@ class PreferencesDialog(Dialog):
 		name_map = {}
 		setattr(self, '%s_map' % type, name_map)
 
-		combobox = self.inputs[type]
 		for app in apps:
 			name = app.name
 			name_map[name] = app.key
-			combobox.append_text(name)
+			widget.append_text(name)
 
+		widget.append_text(self.OTHER_APP)
+		widget.connect('changed', self._on_combo_changed, type)
+
+		widget.current_app = 0
 		try:
 			active = [app.key for app in apps].index(current)
-			combobox.set_active(active)
+			widget.current_app = active
+			widget.set_active(active)
 		except ValueError:
 			pass
 
+	def _on_combo_changed(self, combobox, type):
+		name = combobox.get_active_text()
+		if name == self.OTHER_APP:
+			app = NewApplicationDialog(self, type=type).run()
+			if app:
+				# add new application and select it
+				len = combobox.get_model().iter_n_children(None)
+				name = app.name
+				name_map = getattr(self, '%s_map' % type)
+				name_map[name] = app.key
+				combobox.insert_text(len-2, name)
+				combobox.set_active(len-2)
+			else:
+				# dialog was cancelled - set back to current
+				active = combobox.current_app
+				combobox.set_active(active)
+
 	def do_response_ok(self):
-		if self.use_custom_font.get_active():
+		# Get applications
+		for type, name in self.applicationsform.items():
+			name_map = getattr(self, '%s_map' % type)
+			self.ui.preferences['GtkInterface'][type] = name_map.get(name)
+
+		# Get dynamic tabs
+		for form in self.forms.values():
+			for key, value in form.items():
+				section = form.preferences_sections[key]
+				self.ui.preferences[section][key] = value
+
+		# Set font - special case, consider it a HACK
+		custom = self.ui.preferences['GtkInterface'].pop('use_custom_font')
+		if custom:
 			font = self.fontbutton.get_font_name()
 		else:
 			font = None
 		PageView.style['TextView']['font'] = font
 		PageView.style.write()
 
-		for type in ('file_browser', 'web_browser', 'email_client'):
-			combobox = self.inputs.pop(type)
-			name = combobox.get_active_text()
-			name_map = getattr(self, '%s_map' % type)
-			self.ui.preferences['GtkInterface'][type] = name_map[name]
-
-		fields = self.get_fields()
-		#~ print fields
-		for key, value in fields.items():
-			section, key = key
-			self.ui.preferences[section][key] = value
+		# Save all
 		self.ui.save_preferences()
 		return True
 
@@ -320,15 +350,22 @@ class PluginConfigureDialog(Dialog):
 
 		fields = []
 		self.preferences = dialog.ui.preferences[klass.__name__]
-		for key, type, label, default in klass.plugin_preferences:
-			self.preferences.setdefault(key, default) # just to be sure
-			fields.append((key, type, label, self.preferences[key]))
-		self.add_fields(fields)
+		for pref in klass.plugin_preferences:
+			if len(pref) == 4:
+				key, type, label, default = pref
+				self.preferences.setdefault(key, default) # just to be sure
+			else:
+				key, type, label, default, check = pref
+				self.preferences.setdefault(key, default, check=check) # just to be sure
+
+			fields.append((key, type, label))
+
+		self.add_form(fields, self.preferences)
 
 	def do_response_ok(self):
 		# First let the plugin recieve the changes, then save them.
 		# The plugin could do som conversion on the fly (e.g. Path to string)
-		self.preferences.update(self.get_fields())
+		self.preferences.update(self.form)
 		self.plugin.emit('preferences-changed')
 		self.ui.save_preferences()
 		return True
