@@ -136,6 +136,9 @@ class IndexPath(Path):
 			except IndexError:
 				raise AttributeError, '%s has no attribute %s' % (self.__repr__(), attr)
 
+	def exists(self):
+		return self.haschildren or self.hascontent
+
 	@property
 	def parent(self):
 		'''Returns IndexPath for parent path'''
@@ -562,7 +565,7 @@ class Index(gobject.GObject):
 		#~ print '!! UPDATE LIST', path, path._indexpath
 		assert isinstance(path, IndexPath)
 		if not path.hasdata:
-			path = path.lookup_data(path)
+			path = self.lookup_data(path)
 		hadchildren = path.haschildren
 		hadcontent = path.hascontent
 
@@ -771,7 +774,7 @@ class Index(gobject.GObject):
 			return
 
 		parent = path.parent
-		if isinstance(path, IndexPath):
+		if not isinstance(path, IndexPath):
 			path = self.lookup_data(path)
 			if not path._row:
 				return # apparently it disappeared already
@@ -779,6 +782,7 @@ class Index(gobject.GObject):
 			path = self.lookup_path(path)
 			if not path:
 				self.cleanup(parent) # recurs
+				return # path does not exist in table
 
 		if not (path.hascontent or path.haschildren) \
 		and self.n_list_links(path, direction=LINK_DIR_BACKWARD) == 0:
