@@ -110,6 +110,15 @@ class NotebookList(TextConfigFile):
 		lines = [line + '\n' for line in lines]
 		self.file.writelines(lines)
 
+	def get_default(self):
+		'''Returns uri for the default notebook'''
+		if self.default:
+			return self.default
+		elif len(self) == 1:
+			return self[0]
+		else:
+			return None
+
 	def get_names(self):
 		'''Generator function that yield tuples with the notebook
 		name and the notebook path.
@@ -120,6 +129,7 @@ class NotebookList(TextConfigFile):
 				yield (name, path)
 
 	def get_name(self, uri):
+		'''Find the name for the notebook at 'uri' '''
 		# TODO support for paths that turn out to be files
 		file = Dir(uri).file('notebook.zim')
 		if file.exists():
@@ -129,6 +139,7 @@ class NotebookList(TextConfigFile):
 		return None
 
 	def get_by_name(self, name):
+		'''Get the uri for a notebook'''
 		for n, path in self.get_names():
 			if n.lower() == name.lower():
 				return path
@@ -201,12 +212,7 @@ def resolve_default_notebook():
 	or for the only notebook if there is only a single notebook
 	in the list.
 	'''
-	default = None
-	list = get_notebook_list()
-	if list.default:
-		default = list.default
-	elif len(list) == 1:
-		default = list[0]
+	default = get_notebook_list().get_default()
 
 	if default:
 		if zim.fs.isfile(default):
@@ -248,7 +254,9 @@ def init_notebook(path, name=None):
 	config = ConfigDictFile(path.file('notebook.zim'))
 	config['Notebook']['name'] = name or path.basename
 	config['Notebook']['version'] = '.'.join(map(str, DATA_FORMAT_VERSION))
-	# TODO auto detect if we should enable the slow_fs option
+	if os.name == 'nt': endofline = 'dos'
+	else: endofline = 'unix'
+	config['Notebook']['endofline'] = endofline
 	config.write()
 
 
@@ -369,7 +377,7 @@ class Notebook(gobject.GObject):
 		('home', 'page', _('Home Page')), # T: label for properties dialog
 		('icon', 'image', _('Icon')), # T: label for properties dialog
 		('document_root', 'dir', _('Document Root')), # T: label for properties dialog
-		('slow_fs', 'bool', _('Slow file system')), # T: label for properties dialog
+		('shared', 'bool', _('Shared Notebook')), # T: label for properties dialog
 		#~ ('autosave', 'bool', _('Auto-version when closing the notebook')),
 			# T: label for properties dialog
 	)
@@ -379,7 +387,9 @@ class Notebook(gobject.GObject):
 		gobject.GObject.__init__(self)
 		self._namespaces = []	# list used to resolve stores
 		self._stores = {}		# dict mapping namespaces to stores
-		self.namespace_properties = HierarchicDict()
+		self.namespace_properties = HierarchicDict({
+				'template': 'Default'
+			})
 		self._page_cache = weakref.WeakValueDictionary()
 		self.dir = None
 		self.file = None
@@ -388,17 +398,25 @@ class Notebook(gobject.GObject):
 		self.icon = None
 		self.config = config
 		self.lock = AsyncLock()
+			# We don't use FS.get_async_lock() at this level. A store
+			# backend will automatically trigger this when it calls any
+			# async file operations. This one is more abstract for the
+			# notebook as a whole, regardless of storage
 
 		if dir:
 			assert isinstance(dir, Dir)
 			self.dir = dir
 			self.readonly = not dir.iswritable()
-			self.cache_dir = dir.subdir('.zim')
-			if self.readonly or not self.cache_dir.iswritable():
-				self.cache_dir = self._cache_dir(dir)
-			logger.debug('Cache dir: %s', self.cache_dir)
+
 			if self.config is None:
 				self.config = ConfigDictFile(dir.file('notebook.zim'))
+
+			self.cache_dir = dir.subdir('.zim')
+			if self.readonly or self.config['Notebook'].get('shared') \
+			or not self.cache_dir.iswritable():
+				self.cache_dir = self._cache_dir(dir)
+			logger.debug('Cache dir: %s', self.cache_dir)
+
 			# TODO check if config defined root namespace
 			self.add_store(Path(':'), 'files') # set root
 			# TODO add other namespaces from config
@@ -422,11 +440,15 @@ class Notebook(gobject.GObject):
 			if self.needs_upgrade:
 				logger.warn('This notebook needs to be upgraded to the latest data format')
 
-		self.config['Notebook'].setdefault('name', None, klass=basestring)
-		self.config['Notebook'].setdefault('home', ':Home', klass=basestring)
-		self.config['Notebook'].setdefault('icon', None, klass=basestring)
-		self.config['Notebook'].setdefault('document_root', None, klass=basestring)
-		self.config['Notebook'].setdefault('slow_fs', False)
+		self.config['Notebook'].setdefault('name', None, check=basestring)
+		self.config['Notebook'].setdefault('home', ':Home', check=basestring)
+		self.config['Notebook'].setdefault('icon', None, check=basestring)
+		self.config['Notebook'].setdefault('document_root', None, check=basestring)
+		self.config['Notebook'].setdefault('shared', False)
+		if os.name == 'nt': endofline = 'dos'
+		else: endofline = 'unix'
+		self.config['Notebook'].setdefault('endofline', endofline, check=set(('dos', 'unix')))
+
 		self.do_properties_changed()
 
 	@property
@@ -438,19 +460,29 @@ class Notebook(gobject.GObject):
 		else:
 			return self.file.uri
 
+	@property
+	def endofline(self):
+		return self.config['Notebook']['endofline']
+
 	def _cache_dir(self, dir):
 		from zim.config import XDG_CACHE_HOME
-		path = 'notebook-' + dir.path.replace('/', '_').strip('_')
+		if os.name == 'nt':
+			path = 'notebook-' + dir.path.replace('\\', '_').replace(':', '').strip('_')
+		else:
+			path = 'notebook-' + dir.path.replace('/', '_').strip('_')
 		return XDG_CACHE_HOME.subdir(('zim', path))
 
 	def save_properties(self, **properties):
 		# Check if icon is relative
-		if 'icon' in properties and properties['icon'] \
-		and self.dir and properties['icon'].startswith(self.dir.path):
-			i = len(self.dir.path)
-			path = './' + properties['icon'][i:].lstrip('/\\')
-			# TODO use proper fs routine(s) for this substitution
-			properties['icon'] = path
+		icon = properties.get('icon')
+		if icon:
+			if isinstance(icon, basestring):
+				icon = File(icon)
+
+			if self.dir and icon.ischild(self.dir):
+				properties['icon'] = icon.relpath(self.dir)
+			else:
+				properties['icon'] = icon.path
 
 		# Set home page as string
 		if 'home' in properties and isinstance(properties['home'], Path):
@@ -483,8 +515,7 @@ class Notebook(gobject.GObject):
 		else:
 			self.icon = None
 
-		# Set FS property
-		if config['slow_fs']: print 'TODO: hook slow_fs property'
+		# TODO - can we switch cache_dir on run time when 'shared' chagned ?
 
 	def add_store(self, path, store, **args):
 		'''Add a store to the notebook to handle a specific path and all
@@ -1059,11 +1090,10 @@ class Notebook(gobject.GObject):
 
 		self.move_page(path, newpath, update_links, callback)
 		if update_heading:
-			import zim.parsing
 			page = self.get_page(newpath)
 			tree = page.get_parsetree()
 			if not tree is None:
-				tree.set_heading(zim.parsing.title(newbasename))
+				tree.set_heading(newbasename)
 				page.set_parsetree(tree)
 				self.store_page(page)
 
@@ -1150,24 +1180,25 @@ class Notebook(gobject.GObject):
 
 		page.set_parsetree(tree)
 
-	def resolve_file(self, filename, path):
-		'''Resolves a file or directory path relative to a page. Returns a
-		File object. However the file does not have to exist.
+	def resolve_file(self, filename, path=None):
+		'''Resolves a file or directory path relative to a page or
+		Notebook. Returns a File object. However the file does not
+		have to exist.
 
-		File urls and paths that start with '~/' or '~user/' are considered
-		absolute paths and are returned unmodified.
+		File urls and paths that start with '~/' or '~user/' are
+		considered absolute paths and the corresponding File objects
+		are returned. Also handles windows absolute paths.
 
 		In case the file path starts with '/' the the path is taken relative
 		to the document root - this can e.g. be a parent directory of the
-		notebook. Defaults to the home dir.
+		notebook. Defaults to the filesystem root when no document root
+		is set.
 
 		Other paths are considered attachments and are resolved relative
-		to the namespce below the page.
-
-		Because this is used to resolve file links and is supposed to be
-		platform independent it tries to convert windows filenames to
-		unix equivalents.
+		to the namespace below the page. If no path is given but the
+		notebook has a root folder, this folder is used as base path.
 		'''
+		assert isinstance(filename, basestring)
 		filename = filename.replace('\\', '/')
 		if filename.startswith('~') or filename.startswith('file:/'):
 			return File(filename)
@@ -1180,35 +1211,27 @@ class Notebook(gobject.GObject):
 				# make absolute on unix
 			return File(filename)
 		else:
-			# TODO - how to deal with '..' in the middle of the path ?
-			filepath = [p for p in filename.split('/') if len(p) and p != '.']
-			if not filepath: # filename is e.g. "."
-				return self.get_attachments_dir(path)
-			pagepath = path.name.split(':')
-			filename = filepath.pop()
-			while filepath and filepath[0] == '..':
-				if not pagepath:
-					print 'TODO: handle paths relative to notebook but outside notebook dir'
-					return File('/TODO')
-				else:
-					filepath.pop(0)
-					pagepath.pop()
-			pagename = ':'+':'.join(pagepath + filepath)
-			dir = self.get_attachments_dir(Path(pagename))
-			return dir.file(filename)
+			if path:
+				dir = self.get_attachments_dir(path)
+			else:
+				assert self.dir, 'Can not resolve relative path for notebook without root folder'
+				dir = self.dir
+
+			return File((dir, filename))
 
 	def relative_filepath(self, file, path=None):
-		'''Returns a filepath relative to either the documents dir (/xxx), the
-		attachments dir (if a path is given) (./xxx or ../xxx) or the users
-		home dir (~/xxx). Returns None otherwise.
+		'''Returns a filepath relative to either the documents dir
+		(/xxx), the attachments dir (if a path is given) or the notebook
+		folder (./xxx or ../xxx) or the users home dir (~/xxx).
+		Returns None otherwise.
 
-		Intended as the counter part of resolve_file().
-		Typically this function is used to present the user with readable paths
-		or to shorten the paths inserted in the wiki code. It is advised to
+		Intended as the counter part of resolve_file(). Typically this
+		function is used to present the user with readable paths or to
+		shorten the paths inserted in the wiki code. It is advised to
 		use file uris for links that can not be made relative.
 		'''
+		root = self.dir
 		if path:
-			root = self.dir
 			dir = self.get_attachments_dir(path)
 			if file.ischild(dir):
 				return './'+file.relpath(dir)
@@ -1218,6 +1241,8 @@ class Notebook(gobject.GObject):
 				downpath = file.relpath(parent)
 				up = 1 + uppath.count('/')
 				return '../'*up + downpath
+		elif root and file.ischild(root):
+				return './'+file.relpath(root)
 
 		dir = self.get_document_root()
 		if dir and file.ischild(dir):
@@ -1247,7 +1272,7 @@ class Notebook(gobject.GObject):
 		content for a new page.
 		'''
 		from zim.templates import get_template
-		template = self.namespace_properties[path].get('template', '_New')
+		template = self.namespace_properties[path]['template']
 		logger.debug('Found template \'%s\' for %s', template, path)
 		return get_template('wiki', template)
 
@@ -1392,6 +1417,14 @@ class Path(object):
 		else:
 			self.name = name.strip(':')
 
+		try:
+			self.name = unicode(self.name)
+		except UnicodeDecodeError:
+			raise Error, 'BUG: invalid input, page names should be in ascii, or given as unicode'
+
+	def serialize_zim_config(self):
+		return self.name
+
 	def __repr__(self):
 		return '<%s: %s>' % (self.__class__.__name__, self.name)
 
