@@ -6,18 +6,20 @@ from __future__ import with_statement
 
 import gobject
 import gtk
+import pango
 import logging
 import re
-import datetime
 
+import zim.datetimetz as datetime
 from zim.parsing import parse_date
 from zim.plugins import PluginClass
 from zim.notebook import Path
 from zim.gui.widgets import ui_environment, gtk_get_style,\
 	Dialog, MessageDialog, \
-	Button, IconButton, MenuButton, \
+	InputEntry, Button, IconButton, MenuButton, \
 	BrowserTreeView, SingleClickTreeView
 from zim.formats import get_format, UNCHECKED_BOX, CHECKED_BOX, XCHECKED_BOX
+from zim.config import check_class_allow_empty
 
 
 logger = logging.getLogger('zim.plugins.tasklist')
@@ -95,9 +97,14 @@ This is a core plugin shipping with zim.
 		# key, type, label, default
 		('all_checkboxes', 'bool', _('Consider all checkboxes as tasks'), True),
 			# T: label for plugin preferences dialog
-		('labels', 'string', _('Labels marking tasks'), 'FIXME, TODO'),
+		('tag_by_page', 'bool', _('Turn page name into tags for task items'), False),
+			# T: label for plugin preferences dialog
+		('labels', 'string', _('Labels marking tasks'), 'FIXME, TODO', check_class_allow_empty),
 			# T: label for plugin preferences dialog - labels are e.g. "FIXME", "TODO", "TASKS"
 	)
+	_rebuild_on_preferences = ['all_checkboxes', 'labels']
+		# Rebuild database table if any of these preferences changed.
+		# But leave it alone if others change.
 
 	def __init__(self, ui):
 		PluginClass.__init__(self, ui)
@@ -129,11 +136,14 @@ This is a core plugin shipping with zim.
 		self.db_initialized = True
 
 	def do_preferences_changed(self):
-		self._drop_table()
+		new_preferences = self._serialize_rebuild_on_preferences()
+		if new_preferences != self._current_preferences:
+			self._drop_table()
 		self._set_preferences()
 
 	def _set_preferences(self):
-		self.all_checkboxes = self.preferences['all_checkboxes']
+		self._current_preferences = self._serialize_rebuild_on_preferences()
+
 		string = self.preferences['labels'].strip(' ,')
 		if string:
 			self.task_labels = [s.strip() for s in self.preferences['labels'].split(',')]
@@ -142,6 +152,13 @@ This is a core plugin shipping with zim.
 		regex = '(' + '|'.join(map(re.escape, self.task_labels)) + ')'
 		self.task_label_re = re.compile(regex)
 
+	def _serialize_rebuild_on_preferences(self):
+		# string mapping settings that influence building the table
+		string = ''
+		for pref in self._rebuild_on_preferences:
+			string += str(self.preferences[pref])
+		return string
+
 	def disconnect(self):
 		self._drop_table()
 		PluginClass.disconnect(self)
@@ -207,6 +224,7 @@ This is a core plugin shipping with zim.
 		for node in parsetree.findall('p'):
 			lines = self._flatten_para(node)
 			# Check first line for task list header
+			istasklist = False
 			globaltags = []
 			if len(lines) >= 2 \
 			and isinstance(lines[0], basestring) \
@@ -222,12 +240,13 @@ This is a core plugin shipping with zim.
 				else:
 					# no break occured - all OK
 					lines.pop(0)
+					istasklist = True
 
 			# Check line by line
 			for item in lines:
 				if isinstance(item, tuple):
 					# checkbox
-					if self.all_checkboxes \
+					if istasklist or self.preferences['all_checkboxes'] \
 					or (self.task_labels and self.task_label_re.match(item[2])):
 						open = item[0] == UNCHECKED_BOX
 						tasks.append(self._parse_task(item[2], level=item[1], open=open, tags=globaltags))
@@ -246,7 +265,9 @@ This is a core plugin shipping with zim.
 
 		text = para.text or ''
 		for child in para.getchildren():
-			if child.tag == 'ul':
+			if child.tag == 'strike':
+				continue # Ignore strike out text
+			elif child.tag == 'ul':
 				if text:
 					items += text.splitlines()
 				items += self._flatten_list(child)
@@ -406,7 +427,7 @@ class TaskListDialog(Dialog):
 
 		# Filter input
 		hbox.pack_start(gtk.Label(_('Filter')+': '), False) # T: Input label
-		filter_entry = gtk.Entry()
+		filter_entry = InputEntry()
 		hbox.pack_start(filter_entry, False)
 		clear_button = IconButton('gtk-clear')
 		hbox.pack_start(clear_button, False)
@@ -553,23 +574,6 @@ class TaskListTreeView(BrowserTreeView):
 		self.ui = ui
 		self.plugin = plugin
 
-		cell_renderer = gtk.CellRendererText()
-		for name, i in (
-			(_('Task'), self.TASK_COL), # T: Column header Task List dialog
-			(_('Page'), self.PAGE_COL), # T: Column header Task List dialog
-		):
-			column = gtk.TreeViewColumn(name, cell_renderer, text=i)
-			column.set_resizable(True)
-			column.set_sort_column_id(i)
-			if i == self.TASK_COL:
-				column.set_expand(True)
-				if ui_environment['platform'] == 'maemo':
-					column.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
-					column.set_fixed_width(250)
-					# FIXME probably should also limit the size of this
-					# column on other platforms ...
-			self.append_column(column)
-
 		# Add some rendering for the Prio column
 		def render_prio(col, cell, model, i):
 			prio = model.get_value(i, self.PRIO_COL)
@@ -585,7 +589,22 @@ class TaskListTreeView(BrowserTreeView):
 			# T: Column header Task List dialog
 		column.set_cell_data_func(cell_renderer, render_prio)
 		column.set_sort_column_id(self.PRIO_COL)
-		self.insert_column(column, 0)
+		self.append_column(column)
+
+		# Rendering for task description column
+		cell_renderer = gtk.CellRendererText()
+		cell_renderer.set_property('ellipsize', pango.ELLIPSIZE_END)
+		column = gtk.TreeViewColumn(_('Task'), cell_renderer, text=self.TASK_COL)
+				# T: Column header Task List dialog
+		column.set_resizable(True)
+		column.set_sort_column_id(self.TASK_COL)
+		column.set_expand(True)
+		if ui_environment['platform'] == 'maemo':
+			column.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
+			column.set_fixed_width(250)
+			# FIXME probably should also limit the size of this
+			# column on other platforms ...
+		self.append_column(column)
 
 		# Rendering of the Date column
 		today    = str( datetime.date.today() )
@@ -610,17 +629,33 @@ class TaskListTreeView(BrowserTreeView):
 			# T: Column header Task List dialog
 		column.set_cell_data_func(cell_renderer, render_date)
 		column.set_sort_column_id(self.DATE_COL)
-		self.insert_column(column, 2)
+		self.append_column(column)
 
+		# Rendering for page name column
+		cell_renderer = gtk.CellRendererText()
+		column = gtk.TreeViewColumn(_('Page'), cell_renderer, text=self.PAGE_COL)
+				# T: Column header Task List dialog
+		column.set_resizable(True)
+		column.set_sort_column_id(self.PAGE_COL)
+		self.append_column(column)
+
+		# Finalize
 		self.refresh()
 		self.plugin.connect_object('tasklist-changed', self.__class__.refresh, self)
 
 	def refresh(self):
-		self.real_model.clear()
+		self.real_model.clear() # flush
+
+		# First cache + sort tasks to ensure stability of the list
+		rows = list(self.plugin.list_tasks())
 		paths = {}
-		for row in self.plugin.list_tasks():
+		for row in rows:
 			if not row['source'] in paths:
 				paths[row['source']] = self.plugin.get_path(row)
+
+		rows.sort(key=lambda r: paths[r['source']].name)
+
+		for row in rows:
 			path = paths[row['source']]
 			modelrow = [False, row['prio'], row['description'], row['due'], path.name, row['actionable'], row['open']]
 						# VIS_COL, PRIO_COL, TASK_COL, DATE_COL, PAGE_COL, ACT_COL, OPEN_COL
@@ -647,11 +682,19 @@ class TaskListTreeView(BrowserTreeView):
 		tags = set()
 
 		def collect(model, path, iter):
-			# also count hidden rows here
-			desc = model[iter][self.TASK_COL].decode('utf-8')
+			row = model[iter]
+			if not row[self.OPEN_COL]:
+				return # only count open items
+
+			desc = row[self.TASK_COL].decode('utf-8')
 			for match in tag_re.findall(desc):
 				tags.add(match)
 
+			if self.plugin.preferences['tag_by_page']:
+				name = row[self.PAGE_COL].decode('utf-8')
+				for part in name.split(':'):
+					tags.add(part)
+
 		self.real_model.foreach(collect)
 
 		return tags
@@ -680,7 +723,7 @@ class TaskListTreeView(BrowserTreeView):
 
 	def set_tag_filter(self, tags):
 		if tags:
-			self.tag_filter = ["@"+tag.lower() for tag in tags]
+			self.tag_filter = [tag.lower() for tag in tags]
 		else:
 			self.tag_filter = None
 		self._eval_filter()
@@ -722,10 +765,15 @@ class TaskListTreeView(BrowserTreeView):
 		pagename = modelrow[self.PAGE_COL].lower()
 
 		if visible and self.tag_filter:
-			# And any tag should match
+			# And any tag should match (or pagename if tag_by_page)
 			for tag in self.tag_filter:
-				if tag in description:
-					break
+				if self.plugin.preferences['tag_by_page']:
+					if '@'+tag in description \
+					or tag in pagename.split(':'):
+						break # keep visible True
+				else:
+					if '@'+tag in description:
+						break # keep visible True
 			else:
 				visible = False # no tag found
 
@@ -741,9 +789,9 @@ class TaskListTreeView(BrowserTreeView):
 	def do_row_activated(self, path, column):
 		model = self.get_model()
 		page = Path( model[path][self.PAGE_COL] )
-		#~ task = ...
+		task = unicode(model[path][self.TASK_COL])
 		self.ui.open_page(page)
-		#~ self.ui.mainwindow.pageview.search(task) # FIXME
+		self.ui.mainwindow.pageview.view.get_buffer().finder.find(task)
 
 	def do_button_release_event(self, event):
 		'''Handler for button-release-event, triggers popup menu'''
@@ -776,8 +824,8 @@ class TaskListTreeView(BrowserTreeView):
 				for match in tag_re.findall(desc.decode("UTF-8")):
 					tags.add(match)
 					tasks += "Description: %s\nPriority: %s,  Actionable: %s,  Open: %s,  Due: %s\nPath: %s,  Tags: %s\n\n" % (desc, prio, actionable, opn, due, path_name, ", ".join(tags))
-		tasks += "Number of tasks: %s. Exported: %s UTC." \
-				% (len(model), str(datetime.datetime.utcnow())[:-7])
+		tasks += "Number of tasks: %s. Exported: %s" \
+				% (len(model), str(datetime.now()))
 		#~ print tasks
 		gtk.Clipboard().set_text(tasks.decode("UTF-8"))
 
