@@ -155,8 +155,8 @@ class WWWInterface(NotebookInterface):
 				path = '/'
 			elif path == '/favicon.ico':
 				path = '/+icons/favicon.ico'
-			elif path.startswith('/%2B'):
-				path = '/+' + path[4:] # HACK, very local decoding
+			else:
+			    path = urllib.unquote(path)
 
 			if self.notebook is None:
 				raise NoConfigError
@@ -198,7 +198,6 @@ class WWWInterface(NotebookInterface):
 				else:
 					raise PageNotFoundError(path)
 
-				pagename = urllib.unquote(pagename)
 				path = self.notebook.resolve_path(pagename)
 				page = self.notebook.get_page(path)
 				if page.hascontent:
@@ -214,7 +213,7 @@ class WWWInterface(NotebookInterface):
 			if isinstance(error, (WWWError, FileNotFoundError)):
 				logger.error(error.msg)
 				if isinstance(error, FileNotFoundError):
-					error = PageNotFound(path)
+					error = PageNotFoundError(path)
 					# show url path instead of file path
 				if error.headers:
 					header.extend(error.headers)
@@ -268,6 +267,7 @@ class Server(gobject.GObject):
 		self.running = False
 		self.set_port(port)
 		self.public = public
+		self._io_event = None
 
 		import wsgiref.handlers
 		self.handlerclass = wsgiref.handlers.SimpleHandler
@@ -318,7 +318,8 @@ class Server(gobject.GObject):
 		self.socket.bind((hostname, self.port))
 		self.socket.listen(5)
 
-		gobject.io_add_watch(self.socket, gobject.IO_IN,
+		self._io_event = gobject.io_add_watch(
+			self.socket, gobject.IO_IN,
 			lambda *a: self.do_accept_request())
 
 		self.running = True
@@ -329,6 +330,10 @@ class Server(gobject.GObject):
 		if not self.running:
 			return # ignore silently
 
+		if self._io_event:
+			gobject.source_remove(self._io_event)
+			self._io_event = None
+
 		try:
 			self.socket.close()
 		except Exception, error:
